// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include "sim.h"
#include <math.h>

int get_file_len(char *file_name)
{
FILE *file;
if (!(file=fopen(file_name,"r")))
{
	printf("Error opening file %s\n",file_name);
	exit(0);
}
char buffer[1000];

int i;
i=0;
do
{
	fgets(buffer, 1000, file);
i++;
}while(!feof(file));
i--;
fclose(file);
return i;
}

//Taken from mathworld and wikipidia
double dB(double x)
{
double ret;
if (fabs(x)>1e-10)
{
	ret=(exp(x)-1.0-x*exp(x))/(exp(x)-1)/(exp(x)-1);
}
else
{
	ret=-1.0/2.0+x/6.0-pow(x,3.0)/180.0+pow(x,5.0)/5040;
}

return ret;
}


double B(double x)
{
double ret;
if (mod(x)<1e-16)
{
//From mathworld and wikipidia
	ret=1-x/2.0+pow(x,2.0)/12.0-pow(x,4.0)/720.0+pow(x,6.0)/30240.0;
}else
{
	ret=x/(exp(x)-1.0);
}
return ret;
}

double get_I(struct device *in)
{
int band;
int bands=in->bands;
double ret=0.0;
for (band=0;band<bands;band++)
{
ret+=(in->Jn[0][band]+in->Jp[0][band]+in->Jn[in->ymeshpoints-1][band]+in->Jp[in->ymeshpoints-1][band])*(in->xlen*in->zlen)/2.0;
}
return ret;
}

double get_equiv_J(struct device *in)
{
int band;
int bands=in->bands;
double J=0.0;
for (band=0;band<bands;band++)
{
J+=(in->Jn[0][band]+in->Jp[0][band]+in->Jn[in->ymeshpoints-1][band]+in->Jp[in->ymeshpoints-1][band])/2.0;
}
J+=in->Vapplied/in->Rshunt/(in->xlen*in->zlen);
return J;
}

double get_equiv_V(struct device *in)
{
double J=get_equiv_J(in);
double V=J*in->Rcontact*in->xlen*in->zlen+in->Vapplied;
return V;
}

double mod(double x)
{
if (x<0) return -x;
return x;
}

double get_J(struct device *in)
{
int i;
double ret=0.0;
int band;
int bands=in->bands;

for (i=0;i<in->ymeshpoints;i++)
{
	for (band=0;band<bands;band++)
	{
	ret+=(in->Jn[i][band]+in->Jp[i][band]);
	}
}

return ret/((double)in->ymeshpoints);
}


