// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>

#define dos_warn
double max=-1000;
double min=1000;

#define dosmax 		4
static struct dos dosn[dosmax];
static struct dos dosp[dosmax];
int search(double *x,int N,double find)
{
if (N==1) return 0;
int pos=N/2;
int step=N/2;
do
{
	step=step/2 + (step % 2 > 0 ? 1 : 0);

	if (x[pos]>find)
	{
		pos-=step;
	}else
	{
		pos+=step;
	}

	if (pos<=0)
	{
		pos=0;
		break;
	}
	if (pos>=(N-1))
	{
		pos=N-1;
		break;
	}
	if (step==0) break;
	if (x[pos]==find) break;
	if ((x[pos]<=find)&&((x[pos+1]>find))) break;

}while(1);

if (pos==(N-1)) pos=N-2;
return pos;
}

void dos_init()
{
int i;
for (i=0;i<dosmax;i++)
{
dosn[dosmax].used=FALSE;
dosp[dosmax].used=FALSE;
}

}

void dos_free()
{
//printf("DOS FREE\n");
int i;
int ii;
int iii;
for (i=0;i<dosmax;i++)
{
	if (dosn[i].used==TRUE)
	{
		//printf("Freed dosn %d\n",i);
		free(dosn[i].x);
		free(dosn[i].t);
		free(dosn[i].b);

		for (ii=0;ii<dosn[i].tlen;ii++)
		{
			for (iii=0;iii<dosn[i].xlen;iii++)
			{
				free(dosn[i].c[ii][iii]);
				free(dosn[i].trap[ii][iii]);
			}
			free(dosn[i].c[ii]);
			free(dosn[i].w[ii]);
			free(dosn[i].trap[ii]);
			free(dosn[i].mu[ii]);

		}

		free(dosn[i].c);
		free(dosn[i].w);
		free(dosn[i].trap);
		free(dosn[i].mu);

		dosn[i].xlen=0;
		dosn[i].tlen=0;
		dosn[i].blen=0;
		dosn[i].traps=0;

	}

	if (dosp[i].used==TRUE)
	{
		//printf("Freed dosp %d\n",i);
		free(dosp[i].x);
		free(dosp[i].t);
		free(dosp[i].b);

		for (ii=0;ii<dosp[i].tlen;ii++)
		{
			for (iii=0;iii<dosp[i].xlen;iii++)
			{
				free(dosp[i].c[ii][iii]);
				free(dosp[i].trap[ii][iii]);

			}
			free(dosp[i].c[ii]);
			free(dosp[i].w[ii]);
			free(dosp[i].trap[ii]);
			free(dosp[i].mu[ii]);
		}


		free(dosp[i].c);
		free(dosp[i].trap);
		free(dosp[i].mu);
		free(dosp[i].w);
		dosp[i].xlen=0;
		dosp[i].tlen=0;
		dosp[i].blen=0;
		dosp[i].traps=0;

	}
}
//printf("DOS FREE FINISHED\n");
}

void load_dos(struct device *dev,char *namen, char *namep,int type)
{
int i;
double mu;
int traps;
dosp[type].used=TRUE;
dosn[type].used=TRUE;

printf("Loading %s and %s\n",namen,namep);
	FILE *in;
	in=fopen(namen,"r");
	if (in==NULL)
	{
		printf("DOS n file not found\n");
	}

	int t=0;
	int x=0;
	int b=0;
	double n;
	double w;

	fscanf(in,"%d %d %d %d\n",&(dosn[type].xlen),&(dosn[type].tlen),&(dosn[type].blen),&(dosn[type].traps));
	double xsteps=dosn[type].xlen;
	double tsteps=dosn[type].tlen;
	int bands=dosn[type].blen;
	traps=dosn[type].traps;
	dev->bands=bands;
	dosn[type].x=(double *)malloc(sizeof(double)*(int)xsteps);
	dosn[type].t=(double *)malloc(sizeof(double)*(int)tsteps);
	dosn[type].b=(double *)malloc(sizeof(double)*(int)(bands+traps));


	dosn[type].c=(double ***)malloc(sizeof(double **)*(int)dosn[type].tlen);
	dosn[type].trap=(double ***)malloc(sizeof(double **)*(int)dosn[type].tlen);

	dosn[type].w=(double **)malloc(sizeof(double *)*(int)dosn[type].tlen);
	dosn[type].mu=(double **)malloc(sizeof(double *)*(int)dosn[type].tlen);

	for (t=0;t<tsteps;t++)
	{
		dosn[type].c[t]=(double **)malloc(sizeof(double *)*(int)xsteps);
		dosn[type].trap[t]=(double **)malloc(sizeof(double *)*(int)xsteps);

		for (x=0;x<xsteps;x++)
		{
			dosn[type].c[t][x]=(double *)malloc(sizeof(double )*(int)(bands+1));
			dosn[type].trap[t][x]=(double *)malloc(sizeof(double )*(int)(traps));
		}
		dosn[type].w[t]=(double *)malloc(sizeof(double)*(int)xsteps);
		dosn[type].mu[t]=(double *)malloc(sizeof(double)*(int)xsteps);
	}

	for (x=0;x<xsteps;x++)
	{
		fscanf(in,"%le",&(dosn[type].x[x]));
	}

	for (t=0;t<tsteps;t++)
	{
		fscanf(in,"%le",&(dosn[type].t[t]));
	}

	for (b=0;b<(bands+traps);b++)
	{
		fscanf(in,"%le",&(dosn[type].b[b]));
	}

	for (t=0;t<tsteps;t++)
	{
		for (x=0;x<xsteps;x++)
		{

			fscanf(in,"%le %le %le ",&n,&w,&mu);
			dosn[type].c[t][x][bands]=n;
			dosn[type].w[t][x]=w;
			dosn[type].mu[t][x]=mu;

			for (i=0;i<traps;i++)
			{
				fscanf(in,"%le ",&n);
				dosn[type].trap[t][x][i]=n;
			}

			for (i=0;i<bands;i++)
			{
				fscanf(in,"%le ",&n);
				dosn[type].c[t][x][i]=n;
			}
		}
	}
	fclose(in);

	in=fopen(namep,"r");
	if (in==NULL)
	{
		printf("DOS p file not found\n");
	}
	double p;


	fscanf(in,"%d %d %d %d\n",&(dosp[type].xlen),&(dosp[type].tlen),&(dosp[type].blen),&(dosp[type].traps));
	xsteps=dosp[type].xlen;
	tsteps=dosp[type].tlen;
	bands=dosp[type].blen;
	traps=dosp[type].traps;
//printf("bands %d %d\n",dosp[type].tlen,bands);
//getchar();
	dosp[type].x=(double *)malloc(sizeof(double)*(int)xsteps);
	dosp[type].t=(double *)malloc(sizeof(double)*(int)tsteps);
	dosp[type].b=(double *)malloc(sizeof(double)*(int)(bands+traps));


	dosp[type].c=(double ***)malloc(sizeof(double **)*(int)dosp[type].tlen);
	dosp[type].trap=(double ***)malloc(sizeof(double **)*(int)dosp[type].tlen);

	dosp[type].w=(double **)malloc(sizeof(double *)*(int)dosp[type].tlen);
	dosp[type].mu=(double **)malloc(sizeof(double *)*(int)dosp[type].tlen);

	for (t=0;t<tsteps;t++)
	{
		dosp[type].c[t]=(double **)malloc(sizeof(double *)*(int)xsteps);
		dosp[type].trap[t]=(double **)malloc(sizeof(double *)*(int)xsteps);

		for (x=0;x<xsteps;x++)
		{
			dosp[type].c[t][x]=(double *)malloc(sizeof(double )*(int)(bands+1));
			dosp[type].trap[t][x]=(double *)malloc(sizeof(double )*(int)(traps));
		}

		dosp[type].w[t]=(double *)malloc(sizeof(double)*(int)xsteps);
		dosp[type].mu[t]=(double *)malloc(sizeof(double)*(int)xsteps);

	}

	for (x=0;x<xsteps;x++)
	{
		fscanf(in,"%le",&(dosp[type].x[x]));
	}

	for (t=0;t<tsteps;t++)
	{
		fscanf(in,"%le",&(dosp[type].t[t]));
	}

	for (b=0;b<(bands+traps);b++)
	{
		fscanf(in,"%le",&(dosp[type].b[b]));
	}

	for (t=0;t<tsteps;t++)
	{
		for (x=0;x<xsteps;x++)
		{

			fscanf(in,"%le %le %le",&p,&w,&mu);
			dosp[type].c[t][x][bands]=p;
			dosp[type].w[t][x]=w;
			dosp[type].mu[t][x]=mu;

			for (i=0;i<traps;i++)
			{
				fscanf(in,"%le",&n);
				dosp[type].trap[t][x][i]=n;
			}

			for (i=0;i<bands;i++)
			{
				fscanf(in,"%le",&n);
				dosp[type].c[t][x][i]=n;
			}

			//printf("\n");
			//getchar();

		}
	}
	fclose(in);

	printf("Read DOS....\n");
}


double get_n_w(double top,double T,double m,int type)
{

double ret=0.0;
if (type==0)
{
	ret=(3.0/2.0)*kb*T;
}else
{
double c0=0.0;
double c1=0.0;
double x0=0.0;
double x1=0.0;
double t0=0.0;
double t1=0.0;
double c=0.0;
double xr=0.0;
double tr=0.0;
double c00=0.0;
double c01=0.0;
double c10=0.0;
double c11=0.0;


int t;
int x;

#ifdef dos_warn
if ((dosn[type].x[0]>top)||(dosn[type].x[dosn[type].xlen-1]<top))
{
	printf("Electrons Asking for %e but range %e %e\n",top,dosn[type].x[0],dosn[type].x[dosn[type].xlen-1]);
	//exit(0);
}
#endif

t=search(dosn[type].t,dosn[type].tlen,T);
x=search(dosn[type].x,dosn[type].xlen,top);


if (x<0) x=0;
if (t<0) t=0;

x0=dosn[type].x[x];
x1=dosn[type].x[x+1];
xr=(top-x0)/(x1-x0);
t0=dosn[type].t[t];
t1=dosn[type].t[t+1];
tr=(T-t0)/(t1-t0);

c00=dosn[type].w[t][x];
c01=dosn[type].w[t][x+1];
c10=dosn[type].w[t+1][x];
c11=dosn[type].w[t+1][x+1];

c0=c00+xr*(c01-c00);
c1=c10+xr*(c11-c10);
c=c0+tr*(c1-c0);
ret=c;
}
return ret;
}

double get_p_w(double top,double T,double m,int type)
{
if (top>max) max=top;
if (top<min) min=top;
//printf("%e %e\n",min,max);

double ret=0.0;
if (type==0)
{
	ret=(3.0/2.0)*kb*T;
}else
{
	double c0=0.0;
	double c1=0.0;
	double x0=0.0;
	double x1=0.0;
	double t0=0.0;
	double t1=0.0;
	double c=0.0;
	double xr=0.0;
	double tr=0.0;
	double c00=0.0;
	double c01=0.0;
	double c10=0.0;
	double c11=0.0;


	int t;
	int x;

	#ifdef dos_warn
	if ((dosp[type].x[0]>top)||(dosp[type].x[dosp[type].xlen-1]<top))
	{
		printf("Holes Asking for %e but range %e %e\n",top,dosp[type].x[0],dosp[type].x[dosp[type].xlen-1]);
		//exit(0);
	}
	#endif

/*	for (t=0;t<dosp[type].tlen-1;t++)
	{
	if (dosp[type].t[t]>T) break;
	}

	for (x=0;x<dosp[type].xlen-1;x++)
	{
	if (dosp[type].x[x]>top) break;
	}
	x--;
	t--;*/


//	if (x<0) x=0;
//	if (t<0) t=0;

	t=search(dosp[type].t,dosp[type].tlen,T);
	x=search(dosp[type].x,dosp[type].xlen,top);

	x0=dosp[type].x[x];
	x1=dosp[type].x[x+1];

	xr=(top-x0)/(x1-x0);
	if (xr>1) xr=1;
	t0=dosp[type].t[t];
	t1=dosp[type].t[t+1];
	tr=(T-t0)/(t1-t0);


	c00=dosp[type].w[t][x];
	c01=dosp[type].w[t][x+1];
	c10=dosp[type].w[t+1][x];
	c11=dosp[type].w[t+1][x+1];

	c0=c00+xr*(c01-c00);
	c1=c10+xr*(c11-c10);
	c=c0+tr*(c1-c0);
	ret=c;
}

return ret;
}

double get_top_from_n(double n,double T,double m,double Nc,int type,int band)
{

double ret=0.0;
double c0=0.0;
double c1=0.0;
double x0=0.0;
double x1=0.0;
double cll=0.0;
double crr=0.0;
int xx=0;
double xr=0.0;

double c=0.0;
int t=0;
int x=0;
int bands=dosn[type].blen;
if (band==dos_all) band=bands;

if (type==0)
{
	ret=(kb*T/q)*log((fabs(n))/Nc);
}else
{


//if (dosn[type].tlen>1)
//{
	for (t=0;t<dosn[type].tlen-1;t++)
	{
		if (dosn[type].t[t]>T) break;
	}
	t--;

	for (xx=0;xx<dosn[type].xlen-1;xx++)
	{
		if (dosn[type].c[t+1][xx][band]>n) break;
	}
	xx--;

//}else
//{
	//t=0;
	xx=0;
//}

for (x=0;x<dosn[type].xlen-1;x++)
{
	if (dosn[type].c[t][x][band]>n) break;
}


x--;


//printf("%d %d %d\n",x,xx,dosn[type].xlen);
//getchar();
if (x<0) x=0;
if (xx<0) xx=0;
if (t<0) t=0;

x0=dosn[type].c[t][x][band];
x1=dosn[type].c[t][x+1][band];
xr=(n-x0)/(x1-x0);
if (xr>1) xr=1;
c0=dosn[type].x[x];
c1=dosn[type].x[x+1];

cll=c0+xr*(c1-c0);
//printf("cll = %e\n",cll);
//if (dosn[type].tlen>1)
//{

//x0=dosn[type].c[t+1][xx];
//x1=dosn[type].c[t+1][xx+1];
//printf("values= %d %d %d\n",xx,dosn[type].xlen);

if (x0==x1)
{
	xr=0;
}else
{
	xr=(n-x0)/(x1-x0);
}
c0=dosn[type].x[xx];
c1=dosn[type].x[xx+1];
crr=c0+xr*(c1-c0);

//printf("crr = %e %e %e %e %e %e %e\n",cll,crr,xr,(crr-cll),(n-x0),dosn[type].c[t+1][xx+1],dosn[type].c[t+1][xx]);


	c=cll+(crr-cll)*(T-dosn[type].t[t])/(dosn[type].t[t+1]-dosn[type].t[t]);
//}else
//{
	ret=cll;
//}

//ret=c;

}
//printf("ret = %e\n",ret);
return ret;
}


double get_top_from_p(double p,double T,double m,double Nc,int type, int band)
{
double ret=0.0;
int bands=dosp[type].blen;
if (band==dos_all) band=bands;

if (type==0)
{
	ret=(kb*T/q)*log((fabs(p))/Nc);
}else
{
double c0=0.0;
double c1=0.0;
double x0=0.0;
double x1=0.0;
double cll=0.0;
int xx=0;
double xr=0.0;

double c;
int t;
int x;

//if  (dosp[type].tlen!=1)
//{
	for (t=0;t<dosp[type].tlen-1;t++)
	{
	if (dosp[type].t[t]>T) break;
	}
	t--;

	//for (xx=0;xx<dosp[type].xlen-1;xx++)
	//{
	//if (dosp[type].c[t+1][xx]>p) break;

	//}
	//xx--;

//}else
//{
	//t=0;
	xx=0;
//}

for (x=0;x<dosp[type].xlen-1;x++)
{
if (dosp[type].c[t][x][band]>p) break;
}
x--;
	//printf("Found between %e %e %e\n",p,dosp[type].c[t][x],dosp[type].c[t][x+1]);



if (x<0) x=0;
if (xx<0) xx=0;
if (t<0) t=0;

x0=dosp[type].c[t][x][band];
x1=dosp[type].c[t][x+1][band];
xr=(p-x0)/(x1-x0);
c0=dosp[type].x[x];
c1=dosp[type].x[x+1];
if (xr>1) xr=1;
cll=c0+xr*(c1-c0);

	//if (dosp[type].tlen>1)
	//{
	//	x0=dosp[type].c[t+1][xx];
	//	x1=dosp[type].c[t+1][xx+1];
	//	xr=(p-x0)/(x1-x0);
	//	c0=dosp[type].x[xx];
	//	c1=dosp[type].x[xx+1];
	//	crr=c0+xr*(c1-c0);

	//	c=cll+(crr-cll)*(T-dosp[type].t[t])/(dosp[type].t[t+1]-dosp[type].t[t]);
	//}else
	//{
		c=cll;
	//}
ret=c;
}
return ret;
}

double get_n_den(double top,double T,double m,int type,int band)
{
double ret=0.0;

int bands=dosn[type].blen;
if (band==dos_all) band=bands;

if (type==0)
{
	double N=2.0*pow(((2.0*pi*kb*T*m*m0)/(hp*hp)),1.5);
	ret=N*exp((top*q)/(kb*T));
}else
{
double c0=0.0;
double c1=0.0;
double x0=0.0;
double x1=0.0;
double t0=0.0;
double t1=0.0;
double c=0.0;
double xr=0.0;
double tr=0.0;
double c00=0.0;
double c01=0.0;
double c10=0.0;
double c11=0.0;
int t=0;
int x=0;

#ifdef dos_warn
if ((dosn[type].x[0]>top)||(dosn[type].x[dosn[type].xlen-1]<top))
{
	printf("Electrons Asking for %e but range %e %e\n",top,dosn[type].x[0],dosn[type].x[dosn[type].xlen-1]);
	//exit(0);
}
#endif

t=search(dosn[type].t,dosn[type].tlen,T);
x=search(dosn[type].x,dosn[type].xlen,top);

if (x<0) x=0;
if (t<0) t=0;

x0=dosn[type].x[x];
x1=dosn[type].x[x+1];
xr=(top-x0)/(x1-x0);
t0=dosn[type].t[t];
t1=dosn[type].t[t+1];
tr=(T-t0)/(t1-t0);

c00=dosn[type].c[t][x][band];
c01=dosn[type].c[t][x+1][band];
c10=dosn[type].c[t+1][x][band];
c11=dosn[type].c[t+1][x+1][band];

c0=c00+xr*(c01-c00);
c1=c10+xr*(c11-c10);
c=c0+tr*(c1-c0);
ret=c;
}

return ret;
}

double get_n_trap_den(double top,double T,double m,int type,int band)
{
double ret=0.0;

int bands=dosn[type].blen;
if (band==dos_all) band=bands;

if (type==0)
{
	double N=2.0*pow(((2.0*pi*kb*T*m*m0)/(hp*hp)),1.5);
	ret=N*exp((top*q)/(kb*T));
}else
{
double c0=0.0;
double c1=0.0;
double x0=0.0;
double x1=0.0;
double t0=0.0;
double t1=0.0;
double c=0.0;
double xr=0.0;
double tr=0.0;
double c00=0.0;
double c01=0.0;
double c10=0.0;
double c11=0.0;
int t=0;
int x=0;

#ifdef dos_warn
if ((dosn[type].x[0]>top)||(dosn[type].x[dosn[type].xlen-1]<top))
{
	printf("Electrons Asking for %e but range %e %e\n",top,dosn[type].x[0],dosn[type].x[dosn[type].xlen-1]);
	//exit(0);
}
#endif

t=search(dosn[type].t,dosn[type].tlen,T);
x=search(dosn[type].x,dosn[type].xlen,top);

if (x<0) x=0;
if (t<0) t=0;

x0=dosn[type].x[x];
x1=dosn[type].x[x+1];
xr=(top-x0)/(x1-x0);
t0=dosn[type].t[t];
t1=dosn[type].t[t+1];
tr=(T-t0)/(t1-t0);

c00=dosn[type].trap[t][x][band];
c01=dosn[type].trap[t][x+1][band];
c10=dosn[type].trap[t+1][x][band];
c11=dosn[type].trap[t+1][x+1][band];

c0=c00+xr*(c01-c00);
c1=c10+xr*(c11-c10);
c=c0+tr*(c1-c0);
ret=c;
}

return ret;
}


double get_n_mu(double top,double T,double m,int type)
{
double ret=0.0;

if (type==0)
{
	double N=2.0*pow(((2.0*pi*kb*T*m*m0)/(hp*hp)),1.5);
	ret=N*exp((top*q)/(kb*T));
}else
{
double c0=0.0;
double c1=0.0;
double x0=0.0;
double x1=0.0;
double t0=0.0;
double t1=0.0;
double c=0.0;
double xr=0.0;
double tr=0.0;
double c00=0.0;
double c01=0.0;
double c10=0.0;
double c11=0.0;
int t=0;
int x=0;

#ifdef dos_warn
if ((dosn[type].x[0]>top)||(dosn[type].x[dosn[type].xlen-1]<top))
{
	printf("Electrons Asking for %e but range %e %e\n",top,dosn[type].x[0],dosn[type].x[dosn[type].xlen-1]);
	//exit(0);
}
#endif

t=search(dosn[type].t,dosn[type].tlen,T);
x=search(dosn[type].x,dosn[type].xlen,top);

if (x<0) x=0;
if (t<0) t=0;

x0=dosn[type].x[x];
x1=dosn[type].x[x+1];
xr=(top-x0)/(x1-x0);
t0=dosn[type].t[t];
t1=dosn[type].t[t+1];
tr=(T-t0)/(t1-t0);

c00=dosn[type].mu[t][x];
c01=dosn[type].mu[t][x+1];
c10=dosn[type].mu[t+1][x];
c11=dosn[type].mu[t+1][x+1];

c0=c00+xr*(c01-c00);
c1=c10+xr*(c11-c10);
c=c0+tr*(c1-c0);
ret=c;
}

return ret;
}

double get_dndT_den(double top,double T,double m,int type,int band)
{
int bands=dosn[type].blen;
if (band==dos_all) band=bands;

double ret=0.0;
if (type==0)
{
	double N=2.0*pow(((2.0*pi*kb*T*m*m0)/(hp*hp)),1.5);
	ret=-((top*q)/kb)*N*exp((top*q)/(kb*T))*pow(T,-2.0);
}else
{
	double c0=0.0;
	double c1=0.0;
	double x0=0.0;
	double x1=0.0;
	double t0=0.0;
	double t1=0.0;
	double c=0.0;
	double xr=0.0;
	double tr=0.0;
	double c00=0.0;
	double c01=0.0;
	double c10=0.0;
	double c11=0.0;


	int t;
	int x;

	#ifdef dos_warn
	if ((dosn[type].x[0]>top)||(dosn[type].x[dosn[type].xlen-1]<top))
	{
		printf("Electrons Asking for %e but range %e %e\n",top,dosn[type].x[0],dosn[type].x[dosn[type].xlen-1]);
		//exit(0);
	}
	#endif

	t=search(dosn[type].t,dosn[type].tlen,T);
	x=search(dosn[type].x,dosn[type].xlen,top);

	if (x<0) x=0;
	if (t<0) t=0;

	x0=dosn[type].x[x];
	x1=dosn[type].x[x+1];

	xr=(top-x0)/(x1-x0);
	t0=dosn[type].t[t];
	t1=dosn[type].t[t+1];
	tr=1.0/(t1-t0);


	c00=dosn[type].c[t][x][band];
	c01=dosn[type].c[t][x+1][band];
	c10=dosn[type].c[t+1][x][band];
	c11=dosn[type].c[t+1][x+1][band];

	c0=c00+xr*(c01-c00);
	c1=c10+xr*(c11-c10);
	c=tr*(c1-c0);
	ret=c;
}

return ret;
}

double get_dn_den(double top,double T,double m,int type,int band)
{
int bands=dosn[type].blen;
if (band==dos_all) band=bands;

double ret=0.0;
if (type==0)
{
	double N=2.0*pow(((2.0*pi*kb*T*m*m0)/(hp*hp)),1.5);
	ret=(q/(kb*T))*N*exp((top*q)/(kb*T));
}else
{
	double c0=0.0;
	double c1=0.0;
	double x0=0.0;
	double x1=0.0;
	double t0=0.0;
	double t1=0.0;
	double c=0.0;
	double xr=0.0;
	double tr=0.0;
	double c00=0.0;
	double c01=0.0;
	double c10=0.0;
	double c11=0.0;


	int t;
	int x;

	#ifdef dos_warn
	if ((dosn[type].x[0]>top)||(dosn[type].x[dosn[type].xlen-1]<top))
	{
		printf("Electrons Asking for %e but range %e %e\n",top,dosn[type].x[0],dosn[type].x[dosn[type].xlen-1]);
		//exit(0);
	}
	#endif

	t=search(dosn[type].t,dosn[type].tlen,T);
	x=search(dosn[type].x,dosn[type].xlen,top);

	x0=dosn[type].x[x];
	x1=dosn[type].x[x+1];

	xr=1.0/(x1-x0);
	t0=dosn[type].t[t];
	t1=dosn[type].t[t+1];
	tr=(T-t0)/(t1-t0);

//printf("%d %d %d %d\n",t,x,dosn.tlen,dosn.xlen);
	c00=dosn[type].c[t][x][band];
	c01=dosn[type].c[t][x+1][band];
	c10=dosn[type].c[t+1][x][band];
	c11=dosn[type].c[t+1][x+1][band];

	c0=xr*(c01-c00);
	c1=xr*(c11-c10);
	c=c0+tr*(c1-c0);

	ret=c;
}

return ret;
}

double get_dn_trap_den(double top,double T,double m,int type,int band)
{
int bands=dosn[type].blen;
if (band==dos_all) band=bands;

double ret=0.0;
if (type==0)
{
	double N=2.0*pow(((2.0*pi*kb*T*m*m0)/(hp*hp)),1.5);
	ret=(q/(kb*T))*N*exp((top*q)/(kb*T));
}else
{
	double c0=0.0;
	double c1=0.0;
	double x0=0.0;
	double x1=0.0;
	double t0=0.0;
	double t1=0.0;
	double c=0.0;
	double xr=0.0;
	double tr=0.0;
	double c00=0.0;
	double c01=0.0;
	double c10=0.0;
	double c11=0.0;


	int t;
	int x;

	#ifdef dos_warn
	if ((dosn[type].x[0]>top)||(dosn[type].x[dosn[type].xlen-1]<top))
	{
		printf("Electrons Asking for %e but range %e %e\n",top,dosn[type].x[0],dosn[type].x[dosn[type].xlen-1]);
		//exit(0);
	}
	#endif

	t=search(dosn[type].t,dosn[type].tlen,T);
	x=search(dosn[type].x,dosn[type].xlen,top);

	x0=dosn[type].x[x];
	x1=dosn[type].x[x+1];

	xr=1.0/(x1-x0);
	t0=dosn[type].t[t];
	t1=dosn[type].t[t+1];
	tr=(T-t0)/(t1-t0);

//printf("%d %d %d %d\n",t,x,dosn.tlen,dosn.xlen);
	c00=dosn[type].trap[t][x][band];
	c01=dosn[type].trap[t][x+1][band];
	c10=dosn[type].trap[t+1][x][band];
	c11=dosn[type].trap[t+1][x+1][band];

	c0=xr*(c01-c00);
	c1=xr*(c11-c10);
	c=c0+tr*(c1-c0);

	ret=c;
}

return ret;
}


double get_p_den(double top,double T,double m,int type,int band)
{
int bands=dosp[type].blen;
if (band==dos_all) band=bands;

if (top>max) max=top;
if (top<min) min=top;
//printf("%e %e\n",min,max);

double ret=0.0;
if (type==0)
{
	double N=2.0*pow(((2.0*pi*kb*T*m*m0)/(hp*hp)),1.5);
	ret=N*exp((top*q)/(kb*T));
}else
{
	double c0=0.0;
	double c1=0.0;
	double x0=0.0;
	double x1=0.0;
	double t0=0.0;
	double t1=0.0;
	double c=0.0;
	double xr=0.0;
	double tr=0.0;
	double c00=0.0;
	double c01=0.0;
	double c10=0.0;
	double c11=0.0;


	int t;
	int x;

	#ifdef dos_warn
	if ((dosp[type].x[0]>top)||(dosp[type].x[dosp[type].xlen-1]<top))
	{
		printf("Holes Asking for %e but range %e %e\n",top,dosp[type].x[0],dosp[type].x[dosp[type].xlen-1]);
		//exit(0);
	}
	#endif

	t=search(dosp[type].t,dosp[type].tlen,T);
	x=search(dosp[type].x,dosp[type].xlen,top);

	x0=dosp[type].x[x];
	x1=dosp[type].x[x+1];

	xr=(top-x0)/(x1-x0);
	t0=dosp[type].t[t];
	t1=dosp[type].t[t+1];
	tr=(T-t0)/(t1-t0);


	c00=dosp[type].c[t][x][band];
	c01=dosp[type].c[t][x+1][band];
	c10=dosp[type].c[t+1][x][band];
	c11=dosp[type].c[t+1][x+1][band];

	c0=c00+xr*(c01-c00);
	c1=c10+xr*(c11-c10);
	c=c0+tr*(c1-c0);
	ret=c;
}

return ret;
}

double get_p_mu(double top,double T,double m,int type)
{

if (top>max) max=top;
if (top<min) min=top;
//printf("%e %e\n",min,max);

double ret=0.0;
if (type==0)
{
	double N=2.0*pow(((2.0*pi*kb*T*m*m0)/(hp*hp)),1.5);
	ret=N*exp((top*q)/(kb*T));
}else
{
	double c0=0.0;
	double c1=0.0;
	double x0=0.0;
	double x1=0.0;
	double t0=0.0;
	double t1=0.0;
	double c=0.0;
	double xr=0.0;
	double tr=0.0;
	double c00=0.0;
	double c01=0.0;
	double c10=0.0;
	double c11=0.0;


	int t;
	int x;

	#ifdef dos_warn
	if ((dosp[type].x[0]>top)||(dosp[type].x[dosp[type].xlen-1]<top))
	{
		printf("Holes Asking for %e but range %e %e\n",top,dosp[type].x[0],dosp[type].x[dosp[type].xlen-1]);
		//exit(0);
	}
	#endif

	t=search(dosp[type].t,dosp[type].tlen,T);
	x=search(dosp[type].x,dosp[type].xlen,top);

	x0=dosp[type].x[x];
	x1=dosp[type].x[x+1];

	xr=(top-x0)/(x1-x0);
	t0=dosp[type].t[t];
	t1=dosp[type].t[t+1];
	tr=(T-t0)/(t1-t0);


	c00=dosp[type].mu[t][x];
	c01=dosp[type].mu[t][x+1];
	c10=dosp[type].mu[t+1][x];
	c11=dosp[type].mu[t+1][x+1];

	c0=c00+xr*(c01-c00);
	c1=c10+xr*(c11-c10);
	c=c0+tr*(c1-c0);
	ret=c;
}

return ret;
}

double get_p_trap_den(double top,double T,double m,int type,int band)
{
int bands=dosp[type].blen;
if (band==dos_all) band=bands;

if (top>max) max=top;
if (top<min) min=top;
//printf("%e %e\n",min,max);

double ret=0.0;
if (type==0)
{
	double N=2.0*pow(((2.0*pi*kb*T*m*m0)/(hp*hp)),1.5);
	ret=N*exp((top*q)/(kb*T));
}else
{
	double c0=0.0;
	double c1=0.0;
	double x0=0.0;
	double x1=0.0;
	double t0=0.0;
	double t1=0.0;
	double c=0.0;
	double xr=0.0;
	double tr=0.0;
	double c00=0.0;
	double c01=0.0;
	double c10=0.0;
	double c11=0.0;


	int t;
	int x;

	#ifdef dos_warn
	if ((dosp[type].x[0]>top)||(dosp[type].x[dosp[type].xlen-1]<top))
	{
		printf("Holes Asking for %e but range %e %e\n",top,dosp[type].x[0],dosp[type].x[dosp[type].xlen-1]);
		//exit(0);
	}
	#endif

	t=search(dosp[type].t,dosp[type].tlen,T);
	x=search(dosp[type].x,dosp[type].xlen,top);

	x0=dosp[type].x[x];
	x1=dosp[type].x[x+1];

	xr=(top-x0)/(x1-x0);
	t0=dosp[type].t[t];
	t1=dosp[type].t[t+1];
	tr=(T-t0)/(t1-t0);


	c00=dosp[type].trap[t][x][band];
	c01=dosp[type].trap[t][x+1][band];
	c10=dosp[type].trap[t+1][x][band];
	c11=dosp[type].trap[t+1][x+1][band];

	c0=c00+xr*(c01-c00);
	c1=c10+xr*(c11-c10);
	c=c0+tr*(c1-c0);
	ret=c;
}

return ret;
}


double get_dp_den(double top,double T,double m,int type, int band)
{
int bands=dosp[type].blen;
if (band==dos_all) band=bands;

double ret=0.0;
if (type==0)
{
	double N=2.0*pow(((2.0*pi*kb*T*m*m0)/(hp*hp)),1.5);
	ret=(q/(kb*T))*N*exp((top*q)/(kb*T));
}else
{
	double c0=0.0;
	double c1=0.0;
	double x0=0.0;
	double x1=0.0;
	double t0=0.0;
	double t1=0.0;
	double c=0.0;
	double xr=0.0;
	double tr=0.0;
	double c00=0.0;
	double c01=0.0;
	double c10=0.0;
	double c11=0.0;


	int t;
	int x;

	#ifdef dos_warn
	if ((dosp[type].x[0]>top)||(dosp[type].x[dosp[type].xlen-1]<top))
	{
		printf("Holes Asking for %e but range %e %e\n",top,dosp[type].x[0],dosp[type].x[dosp[type].xlen-1]);
		//exit(0);
	}
	#endif

	t=search(dosp[type].t,dosp[type].tlen,T);
	x=search(dosp[type].x,dosp[type].xlen,top);

	x0=dosp[type].x[x];
	x1=dosp[type].x[x+1];

	xr=1.0/(x1-x0);
	t0=dosp[type].t[t];
	t1=dosp[type].t[t+1];
	tr=(T-t0)/(t1-t0);


	c00=dosp[type].c[t][x][band];
	c01=dosp[type].c[t][x+1][band];
	c10=dosp[type].c[t+1][x][band];
	c11=dosp[type].c[t+1][x+1][band];

	c0=xr*(c01-c00);
	c1=xr*(c11-c10);
	c=c0+tr*(c1-c0);
	ret=c;

}


return ret;
}

double get_dp_trap_den(double top,double T,double m,int type, int band)
{
int bands=dosp[type].blen;
if (band==dos_all) band=bands;

double ret=0.0;
if (type==0)
{
	double N=2.0*pow(((2.0*pi*kb*T*m*m0)/(hp*hp)),1.5);
	ret=(q/(kb*T))*N*exp((top*q)/(kb*T));
}else
{
	double c0=0.0;
	double c1=0.0;
	double x0=0.0;
	double x1=0.0;
	double t0=0.0;
	double t1=0.0;
	double c=0.0;
	double xr=0.0;
	double tr=0.0;
	double c00=0.0;
	double c01=0.0;
	double c10=0.0;
	double c11=0.0;


	int t;
	int x;

	#ifdef dos_warn
	if ((dosp[type].x[0]>top)||(dosp[type].x[dosp[type].xlen-1]<top))
	{
		printf("Holes Asking for %e but range %e %e\n",top,dosp[type].x[0],dosp[type].x[dosp[type].xlen-1]);
		//exit(0);
	}
	#endif

	t=search(dosp[type].t,dosp[type].tlen,T);
	x=search(dosp[type].x,dosp[type].xlen,top);

	x0=dosp[type].x[x];
	x1=dosp[type].x[x+1];

	xr=1.0/(x1-x0);
	t0=dosp[type].t[t];
	t1=dosp[type].t[t+1];
	tr=(T-t0)/(t1-t0);


	c00=dosp[type].trap[t][x][band];
	c01=dosp[type].trap[t][x+1][band];
	c10=dosp[type].trap[t+1][x][band];
	c11=dosp[type].trap[t+1][x+1][band];

	c0=xr*(c01-c00);
	c1=xr*(c11-c10);
	c=c0+tr*(c1-c0);
	ret=c;
}


return ret;
}


double get_dpdT_den(double top,double T,double m,int type, int band)
{
int bands=dosp[type].blen;
if (band==dos_all) band=bands;

double ret=0.0;
if (type==0)
{
	double N=2.0*pow(((2.0*pi*kb*T*m*m0)/(hp*hp)),1.5);
	ret=-((top*q)/kb)*N*exp((top*q)/(kb*T))*pow(T,-2.0);
}else
{
	double c0=0.0;
	double c1=0.0;
	double x0=0.0;
	double x1=0.0;
	double t0=0.0;
	double t1=0.0;
	double c=0.0;
	double xr=0.0;
	double tr=0.0;
	double c00=0.0;
	double c01=0.0;
	double c10=0.0;
	double c11=0.0;

	#ifdef dos_warn
	if ((dosp[type].x[0]>top)||(dosp[type].x[dosp[type].xlen-1]<top))
	{
		printf("Holes Asking for %e but range %e %e\n",top,dosp[type].x[0],dosp[type].x[dosp[type].xlen-1]);
		//exit(0);
	}
	#endif

	int t;
	int x;
	t=search(dosp[type].t,dosp[type].tlen,T);
	x=search(dosp[type].x,dosp[type].xlen,top);

	if (x<0) x=0;
	if (t<0) t=0;

	x0=dosp[type].x[x];
	x1=dosp[type].x[x+1];
	xr=(top-x0)/(x1-x0);
	t0=dosp[type].t[t];
	t1=dosp[type].t[t+1];
	tr=1.0/(t1-t0);

	c00=dosp[type].c[t][x][band];
	c01=dosp[type].c[t][x+1][band];
	c10=dosp[type].c[t+1][x][band];
	c11=dosp[type].c[t+1][x+1][band];

	c0=c00+xr*(c01-c00);
	c1=c10+xr*(c11-c10);
	c=tr*(c1-c0);
	ret=c;
}


return ret;
}

