// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>

void get_initial(struct device *in)
{
printf("get_initial\n");
int i;
double *FnmEc;
double Ef;
double Nad=0.0;
FnmEc = malloc(in->ymeshpoints*sizeof(double));
for (i=0;i<in->ymeshpoints;i++)
{
Nad=in->Nad[i];
double temp=0;
int stats=in->dostype[i];
if (i==0)
{
	temp=in->mat.l[in->ymatlayers[i]].space;
	if (temp!=-1)  Nad=temp;
	stats=in->bdos;
}

if (i==(in->ymeshpoints-1))
{
	temp=in->mat.l[in->ymatlayers[i]].space;//get_mat_param_poly(&(in->mat.l[in->ymatlayers[i]]),mat_br,0.0);
	if (temp!=-1)  Nad=temp;
	stats=in->bdos;
}

	if (Nad>0)	//positave
	{

		FnmEc[i]=-get_top_from_p(fabs(Nad),in->Te[i],in->mh[i],in->Nv[i],stats,dos_all)-in->Eg[i];	//Fn-Ec
		if (i==0)  printf("p= %e %e\n",(kb*in->Te[i]/q)*log(fabs(Nad)/in->Nv[i]),get_top_from_p(fabs(Nad),in->Te[i],in->mh[i],in->Nv[i],stats,dos_all));
		//
	}else
	{
		FnmEc[i]=get_top_from_n(fabs(Nad),in->Te[i],in->me[i],in->Nc[i],stats,dos_all);//(kb*in->Th[i]/q)*log((fabs(Nad))/in->Nc[i]);		//Fn-Ec
		double crash=get_top_from_n(fabs(Nad),in->Te[i],in->me[i],in->Nc[i],stats,dos_all);
		if (i==(in->ymeshpoints-1)) printf("n= %e %e %e\n",(kb*in->Th[i]/q)*log((fabs(Nad))/in->Nc[i]),FnmEc[i],crash);
		//getchar();
		//printf("Gone negitave\n");
	}

	in->phi[i]=FnmEc[i]-FnmEc[0]-in->Xi[i]+in->Xi[0];
	in->Ec[i]=-in->phi[i]-in->Xi[i];
	in->Ev[i]=-in->phi[i]-in->Xi[i]-in->Eg[i];
	in->Ecphoton[i]=-in->phi[i]-in->Xiphoton[i];
	in->Evphoton[i]=-in->phi[i]-in->Xiphoton[i]-in->Egphoton[i];



}
Ef=FnmEc[0]-in->Xi[0]-in->phi[0];
//getchar();
int band;
int bands=in->bands;
for (i=0;i<in->ymeshpoints;i++)
{
	in->Fi[i]=Ef;

	for (band=0;band<bands;band++)
	{
	in->Fn[i][band]=Ef;
	in->Fp[i][band]=Ef;
	}

	for (band=0;band<bands;band++)
	{
		in->n[i][band]=in->Nc[i]*exp(((in->Fi[i]-in->Ec[i])*q)/(kb*in->Te[i]));
		in->p[i][band]=in->Nv[i]*exp(((in->Ev[i]-in->Fi[i])*q)/(kb*in->Th[i]));

		in->mun[i][band]=get_mat_param_mobility(&(in->mat.l[in->ymatlayers[i]]),mat_mue,band);
		in->mup[i][band]=get_mat_param_mobility(&(in->mat.l[in->ymatlayers[i]]),mat_muh,band);
	}
	in->mun_avg[i]=get_mat_param(&(in->mat.l[in->ymatlayers[i]]),mat_mue);
	in->mup_avg[i]=get_mat_param(&(in->mat.l[in->ymatlayers[i]]),mat_muh);


}



//getchar();
//Now get the voltages

in->Vl=in->phi[0];
//double Xil=get_mat_param(&(in->lcontact),mat_Xi);
//double Xir=get_mat_param(&(in->rcontact),mat_Xi);
//double xnr=in->Vbias+in->Fi[i];
//double tnr=in->Xi[in->ymeshpoints-1];


in->deltaFrn=get_top_from_n(fabs(in->mat.l[in->ymatlayers[in->ymeshpoints-1]].space),in->Te[in->ymeshpoints-1],in->me[in->ymeshpoints-1],in->Nc[in->ymeshpoints-1],in->bdos,dos_all);
in->deltaFrp=-in->deltaFrn-in->Eg[in->ymeshpoints-1];

in->deltaFlp=get_top_from_p(in->mat.l[in->ymatlayers[0]].space,in->Th[0],in->mh[0],in->Nv[0],in->bdos,dos_all);
in->deltaFln=-in->deltaFrn-in->Eg[in->ymeshpoints-1];


printf ("Lp = %e %e\n", get_p_den(in->deltaFlp,in->Th[0],in->mh[0],in->bdos,dos_all),in->deltaFlp);
printf ("Ln = %e %e\n", get_n_den(in->deltaFln,in->Te[0],in->me[0],in->bdos,dos_all),in->deltaFln);

printf ("Rp = %e %e\n", get_p_den(in->deltaFrp,in->Th[in->ymeshpoints-1],in->mh[in->ymeshpoints-1],in->bdos,dos_all),in->deltaFrp);
printf ("Rn = %e %e\n", get_n_den(in->deltaFrn,in->Te[in->ymeshpoints-1],in->me[in->ymeshpoints-1],in->bdos,dos_all),in->deltaFrn);
//getchar();

//getchar();
if (in->Vbias==-1)
{
	in->Vr=in->phi[in->ymeshpoints-1];
}else
{
	in->Vr=in->Vbias;
}
//Xil-Xir;
char test[200];
sprintf(test,"Vl=%lf Vr=%lf delta=%lf\n",in->Vl,in->Vr,in->Vr-in->Vl);
printf("%s",test);
//getchar();
printdebug(test);

free(FnmEc);
}

