// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>

void quick_update();

#ifdef only
int only_update_thermal=FALSE;
#endif

void update_solver_vars(struct device *in,int clamp)
{
int i;
int band;
int bands=in->bands;
double update=0.0;
	for (i=0;i<in->ymeshpoints;i++)
	{
#ifdef only
		if (only_update_thermal==FALSE)
		{
#endif
			update=in->b[i];
			if (clamp==TRUE)
			{
				in->phi[i]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));
			}else
			{
				in->phi[i]+=update;

			}

			for (band=0;band<bands;band++)
			{
				update=(in->b[in->ymeshpoints*(1+band)+i]);
				if (clamp==TRUE)
				{
					in->x[i][band]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));
				}else
				{
					in->x[i][band]+=update;
				}
			}


			for (band=0;band<bands;band++)
			{
				update=(in->b[in->ymeshpoints*(1+bands+band)+i]);
				if (clamp==TRUE)
				{
					in->xp[i][band]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));//
				}else
				{
					in->xp[i][band]+=update;

				}
			}

			/*
			update=(in->b[i+3*in->ymeshpoints]);
			if (clamp==TRUE)
			{
				in->ex[i]+=update/(1.0+fabs(update/1e30/(300.0*kb/q)));;
			}else
			{
				in->ex[i]+=update;

			}


		if (in->nnewton==TRUE)
		{
			update=(in->b[in->ymeshpoints*(1+bands+bands)+i]);
			if (clamp==TRUE)
			{
				in->Te[i]+=update/(1.0+fabs(update/in->thermal_clamp/(300.0*kb/q)));
			}else
			{
				in->Te[i]+=update;

			}
		}
*/

#ifdef only

		}else
		{
#endif

		if (in->ntrapnewton==TRUE)
		{
			update=(in->b[in->ymeshpoints*(1+bands+bands)+i]);
			if (clamp==TRUE)
			{
				in->xt[i]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));
			}else
			{
				in->xt[i]+=update;

			}
		}

		if (in->ptrapnewton==TRUE)
		{
			update=(in->b[in->ymeshpoints*(1+bands+bands+1)+i]);
			if (clamp==TRUE)
			{
				in->xpt[i]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));
			}else
			{
				in->xpt[i]+=update;

			}
		}


		if (in->hnewton==TRUE)
		{
			update=(in->b[i+5*in->ymeshpoints]);
			if (clamp==TRUE)
			{
				in->Th[i]+=update/(1.0+fabs(update/in->thermal_clamp/(300.0*kb/q)));
			}else
			{
				in->Th[i]+=update;

			}
		}

		if (in->lnewton==TRUE)
		{
			update=(in->b[i+6*in->ymeshpoints]);
			if (clamp==TRUE)
			{
				in->Tl[i]+=update/(1.0+fabs(update/in->thermal_clamp/(300.0*kb/q)));
			}else
			{
				in->Tl[i]+=update;

			}
		}


#ifdef only

		}
#endif

#ifdef only
	if (only_update_thermal==FALSE)
	{
#endif
		for (band=0;band<in->bands;band++)
		{
			in->Fn[i][band]=in->x[i][band]-in->phi[i];
			in->Fp[i][band]=-in->xp[i][band]-in->phi[i];
		}

		in->Fnt[i]=in->xt[i]-in->phi[i];
		in->Fpt[i]=-in->xpt[i]-in->phi[i];
		//printf("%e\n",in->Fpt[i]);

		in->Ec[i]=-in->phi[i]-in->Xi[i];
		in->Ev[i]=-in->phi[i]-in->Xi[i]-in->Eg[i];
		in->Evphoton[i]=-in->phi[i]-in->Xiphoton[i]-in->Egphoton[i];


		in->mun_avg[i]=0.0;
		in->mup_avg[i]=0.0;

		for (band=0;band<in->bands;band++)
		{
			in->dn[i][band]=get_dn_den(in->x[i][band]+in->t[i][band],in->Te[i],in->me[i],in->dostype[i],dos_all);
			in->n[i][band]=get_n_den(in->x[i][band]+in->t[i][band],in->Te[i],in->me[i],in->dostype[i],dos_all);
			in->dndTe[i][band]=get_dndT_den(in->x[i][band]+in->t[i][band],in->Te[i],in->me[i],in->dostype[i],dos_all);
			in->dp[i][band]=get_dp_den(in->xp[i][band]-in->tp[i][band],in->Th[i],in->mh[i],in->dostype[i],dos_all);
			in->p[i][band]=get_p_den(in->xp[i][band]-in->tp[i][band],in->Th[i],in->mh[i],in->dostype[i],dos_all);
			//if (in->Vapplied<0.1)
			//{
			in->mun[i][band]=get_n_mu(in->x[i][band]+in->t[i][band],in->Te[i],in->me[i],in->dostype[i]);
			in->mup[i][band]=get_p_mu(in->xp[i][band]-in->tp[i][band],in->Th[i],in->mh[i],in->dostype[i]);//get_mat_param_mobility(&(in->mat.l[in->ymatlayers[i]]),mat_muh,band);
			//printf("update\n");
			//}
			//printf("%e %e\n",get_mat_param_mobility(&(in->mat.l[in->ymatlayers[i]]),mat_mue,band),get_n_mu(in->x[i][band]+in->t[i][band],in->Te[i],in->me[i],in->dostype[i]));

			in->mun_avg[i]+=in->mun[i][band];
			in->mup_avg[i]+=in->mup[i][band];

		}

		in->mun_avg[i]/=(double)bands;
		in->mup_avg[i]/=(double)bands;

//printf("%e %e\n",in->mun_avg[i],in->mup_avg[i]);

		if (in->ntrapnewton)
		{
			in->nt[i]=get_n_trap_den(in->xt[i]+in->tt[i],in->Te[i],in->me[i],in->dostype[i],0);
			in->dnt[i]=get_dn_trap_den(in->xt[i]+in->tt[i],in->Te[i],in->me[i],in->dostype[i],0);
		}else if (in->combitrap)
		{
			in->nt[i]=get_n_trap_den(in->x[i][0]+in->t[i][0],in->Te[i],in->me[i],in->dostype[i],0);
			in->dnt[i]=get_dn_trap_den(in->x[i][0]+in->t[i][0],in->Te[i],in->me[i],in->dostype[i],0);
		}else
		{
			in->nt[i]=0.0;
			in->dnt[i]=0.0;
		}


		if (in->ptrapnewton)
		{
			in->pt[i]=get_p_trap_den(in->xpt[i]-in->tpt[i],in->Th[i],in->mh[i],in->dostype[i],0);
			in->dpt[i]=get_dp_trap_den(in->xpt[i]-in->tpt[i],in->Th[i],in->mh[i],in->dostype[i],0);
		}else if (in->combitrap)
		{
			in->pt[i]=get_p_trap_den(in->xp[i][0]-in->tp[i][0],in->Th[i],in->mh[i],in->dostype[i],0);
			in->dpt[i]=get_dp_trap_den(in->xp[i][0]-in->tp[i][0],in->Th[i],in->mh[i],in->dostype[i],0);
		}
		else
		{
			in->dpt[i]=0.0;
			in->pt[i]=0.0;
		}


		in->ni[i]=sqrt(get_n_den(in->Fi[i]-in->Ec[i],in->Te[i],in->me[i],in->dostype[i],dos_all)*get_p_den(in->Ev[i]-in->Fi[i],in->Th[i],in->mh[i],in->dostype[i],dos_all));
//printf("%e\n",sqrt(get_n_den(in->Fi[i]-in->Ec[i],in->Te[i],in->me[i],in->dostype[i],dos_all,0.0)*get_p_den(in->Ev[i]-in->Fi[i],in->Th[i],in->mh[i],in->dostype[i],dos_all)));
		}
		//getchar();

//printf("asad");
//getchar();

#ifdef only
	}
#endif

}

void fill_matrix(struct device *in)
{
int band;
int bands=in->bands;
FILE *file_j =fopen("myj.dat","w");
//getchar();
double phil;
double phic;
double phir;
double yl;
double yc;
double yr;
double dyl;
double dyr;
double ddh;
double dh;
int pos=0;

double Ecl=0.0;
double Ecr=0.0;
double Ecc=0.0;
double Evl=0.0;
double Evr=0.0;
double Evc=0.0;

double Tll=300.0;
double Tlc=300.0;
double Tlr=300.0;

double Tel=300.0;
double Tec=300.0;
double Ter=300.0;

double Thl=300.0;
double Thc=300.0;
double Thr=300.0;

double ni=0.0;

double *Jnl=(double*)malloc(bands*sizeof(double));
double *Jnr=(double*)malloc(bands*sizeof(double));
double *Jpl=(double*)malloc(bands*sizeof(double));
double *Jpr=(double*)malloc(bands*sizeof(double));

double *Dnl=(double*)malloc(bands*sizeof(double));
double *Dnc=(double*)malloc(bands*sizeof(double));
double *Dnr=(double*)malloc(bands*sizeof(double));
double *Dpl=(double*)malloc(bands*sizeof(double));
double *Dpc=(double*)malloc(bands*sizeof(double));
double *Dpr=(double*)malloc(bands*sizeof(double));

double *nl=(double*)malloc(bands*sizeof(double));
double *nc=(double*)malloc(bands*sizeof(double));
double *nr=(double*)malloc(bands*sizeof(double));

double *pl=(double*)malloc(bands*sizeof(double));
double *pc=(double*)malloc(bands*sizeof(double));
double *pr=(double*)malloc(bands*sizeof(double));

double *xil=(double*)malloc(bands*sizeof(double));
double *xir=(double*)malloc(bands*sizeof(double));
double *xipl=(double*)malloc(bands*sizeof(double));
double *xipr=(double*)malloc(bands*sizeof(double));

double *dJpdxipl=(double*)malloc(bands*sizeof(double));
double *dJpdxipc=(double*)malloc(bands*sizeof(double));
double *dJpdxipr=(double*)malloc(bands*sizeof(double));

double *dnl=(double*)malloc(bands*sizeof(double));
double *dnc=(double*)malloc(bands*sizeof(double));
double *dnr=(double*)malloc(bands*sizeof(double));

double *dpl=(double*)malloc(bands*sizeof(double));
double *dpc=(double*)malloc(bands*sizeof(double));
double *dpr=(double*)malloc(bands*sizeof(double));

double *munl=(double*)malloc(bands*sizeof(double));
double *munc=(double*)malloc(bands*sizeof(double));
double *munr=(double*)malloc(bands*sizeof(double));

double *mupl=(double*)malloc(bands*sizeof(double));
double *mupc=(double*)malloc(bands*sizeof(double));
double *mupr=(double*)malloc(bands*sizeof(double));


double *wnl=(double*)malloc(bands*sizeof(double));
double *wnc=(double*)malloc(bands*sizeof(double));
double *wnr=(double*)malloc(bands*sizeof(double));

double *wpl=(double*)malloc(bands*sizeof(double));
double *wpc=(double*)malloc(bands*sizeof(double));
double *wpr=(double*)malloc(bands*sizeof(double));

double *dnldTe=(double*)malloc(bands*sizeof(double));
double *dncdTe=(double*)malloc(bands*sizeof(double));
double *dnrdTe=(double*)malloc(bands*sizeof(double));

double *dpldTh=(double*)malloc(bands*sizeof(double));
double *dpcdTh=(double*)malloc(bands*sizeof(double));
double *dprdTh=(double*)malloc(bands*sizeof(double));

double *nc_all=(double*)malloc(bands*sizeof(double));
double *pc_all=(double*)malloc(bands*sizeof(double));
double *dnc_all=(double*)malloc(bands*sizeof(double));
double *dpc_all=(double*)malloc(bands*sizeof(double));


double *dJdxil=(double*)malloc(bands*sizeof(double));
double *dJdxic=(double*)malloc(bands*sizeof(double));
double *dJdxir=(double*)malloc(bands*sizeof(double));

double *dJdphil=(double*)malloc(bands*sizeof(double));
double *dJdphic=(double*)malloc(bands*sizeof(double));
double *dJdphir=(double*)malloc(bands*sizeof(double));

double *dJpdphil=(double*)malloc(bands*sizeof(double));
double *dJpdphic=(double*)malloc(bands*sizeof(double));
double *dJpdphir=(double*)malloc(bands*sizeof(double));


double *dphidxic=(double*)malloc(bands*sizeof(double));
double *dphidxipc=(double*)malloc(bands*sizeof(double));

double xnr;
double tnr;
double xnl;
double tnl;

double xpr;
double tpr;
double xpl;
double tpl;


double exl;
double exr;
double exc;
double Dexl;
double Dexc;
double Dexr;
double R;

double epr;
double epc;
double epl;

//double dexl;
//double dexc;
//double dexr;
//double dex;
double G;
double Gex;
int i;
double dJdxipc=0.0;
double dJpdxic=0.0;

double mel=0.0;
double mec=0.0;
double mer=0.0;

double mhl=0.0;
double mhc=0.0;
double mhr=0.0;


double e0=0.0;
double e1=0.0;

double dphil=0.0;
double dphic=0.0;
double dphir=0.0;
double Rbi=0.0;
double Rbi_mul=1.0;
double deriv=0.0;

double nlast=0.0;
double plast=0.0;
double dt=0.0;

//double dTel;
//double dTec;
//double dTer;
//double dTe;


//double dJnldTe=0.0;
//double dJncdTe=0.0;
//double dJnrdTe=0.0;
//double dphidTe=0.0;
//double dTeldn=0.0;
//double dTecdn=0.0;
//double dTerdn=0.0;
//double dThl=0.0;
//double dThc=0.0;
//double dThr=0.0;
//double dTh=0.0;

//double dThldp=0.0;
//double dThcdp=0.0;
//double dThrdp=0.0;

double kll=0.0;
double klc=0.0;
double klr=0.0;

double kl0=0.0;
double kl1=0.0;

//double dTll=0.0;
//double dTlc=0.0;
//double dTlr=0.0;
//double dTl=0.0;

//double dTlcdTe=0.0;
//double dTlcdTh=0.0;

//double dTlcdn=0.0;
//double dTlcdp=0.0;

//double dJpldTh=0.0;
//double dJpcdTh=0.0;
//double dJprdTh=0.0;

//double dSnl_predTe=0.0;
//double dSnr_predTe=0.0;

//double dTeldphi=0.0;
//double dTecdphi=0.0;
//double dTerdphi=0.0;

//double Snl=0.0;
//double Snr=0.0;
//double Snl_pre=0.0;
//double Snr_pre=0.0;


//double dsxilpdThl=0.0;
//double dsxilpdThc=0.0;
//double dsxirpdThc=0.0;
//double dsxirpdThr=0.0;

//double sxipl=0.0;
//double sxipr=0.0;

//double Spl=0.0;
//double Spr=0.0;

//double Spl_pre=0.0;
//double Spr_pre=0.0;
//double dSpl_predTh=0.0;
//double dSpr_predTh=0.0;

//double dxildTel=0.0;
//double dxildTec=0.0;
//double dxirdTec=0.0;
//double dxirdTer=0.0;

//double dThldphi=0.0;
//double dThcdphi=0.0;
//double dThrdphi=0.0;

//double dxilpdThl=0.0;
//double dxilpdThc=0.0;

//double dxirpdThc=0.0;
//double dxirpdThr=0.0;


double tauncapc=0.0;
double taunescapec=0.0;
double ntc=0.0;



double taupcapc=0.0;
double taupescapec=0.0;
double ptc=0.0;


int forcecharge=TRUE;
double one;


double dntrapdn=0.0;
double dntrapdntrap=0.0;
double dphidntrap=0.0;
double dntc=0.0;
double dntrap=0.0;

double dptrapdp=0.0;
double dptrapdptrap=0.0;
double dphidptrap=0.0;
double dptc=0.0;
double dptrap=0.0;

double length=in->ymesh[in->ymeshpoints-1];
double n_sum=0.0;
double p_sum=0.0;
double np_sum=0.0;
double l=0.0;
double r=0.0;
double delta=0.0;
double deibel=0.0;
if (in->Rbitype==5)
{
	for (i=0;i<in->ymeshpoints;i++)
	{
		l=in->ymesh[i];
		if (i==in->ymeshpoints-1)
		{
			r=in->ymesh[i]+in->ymesh[i]-in->ymesh[i-1];
		}else
		{
			r=in->ymesh[i+1];
		}

		delta=r-l;

		n_sum+=in->n[i][0]*delta;
		p_sum+=in->p[i][0]*delta;
		np_sum+=in->n[i][0]*in->p[i][0]*delta;

	}
n_sum/=length;
p_sum/=length;
np_sum/=length;
deibel=np_sum/(n_sum*p_sum);
}

calculate_kd(in);


		pos=0;
		for (i=0;i<in->ymeshpoints;i++)
		{

			if (i==0)
			{
				exl=0.0;
				Dexl=in->Dex[0];
				phil=in->Vapplied;
				yl=in->ymesh[0]<-(in->ymesh[1]-in->ymesh[0]);
				mel=in->me[0];
				mhl=in->mh[0];
				Tll=in->Tll;
				Tel=in->Tll;
				Thl=in->Tll;




				Ecl=-in->Xi[0]-phil;
				Evl=-in->Xi[0]-phil-in->Eg[0];
				epl=in->epsilonr[0]*epsilon0;

				for (band=0;band<bands;band++)
				{
					xnl=in->Fi[0];
					tnl=in->Xi[0];
					if (forcecharge==TRUE)
					{
						one=in->deltaFln;
					}else
					{
						one=xnl+tnl;
					}

					nl[band]=get_n_den(one,Tel,mel,in->bdos,dos_all);
					dnl[band]=get_dn_den(one,Tel,mel,in->bdos,dos_all);
					wnl[band]=get_n_w(one,Tel,mel,in->bdos);
					dnldTe[band]=0.0;
					munl[band]=in->mun[0][band];
				}


				for (band=0;band<bands;band++)
				{
					xpl=-in->Fi[0];
					tpl=in->Xi[0]+in->Eg[0];

					if (forcecharge==TRUE)
					{
						one=in->deltaFlp;
					}else
					{
						one=xpl-tpl;
					}

					pl[band]=get_p_den(one,Thl,mhl,in->bdos,dos_all);

					dpl[band]=get_dp_den(one,Thl,mhl,in->bdos,dos_all);
					wpl[band]=get_p_w(one,Thl,mhl,in->bdos);
					dpldTh[band]=0.0;
					mupl[band]=in->mup[0][band];
				}



				kll=in->kl[i];

			}else
			{
				Dexl=in->Dex[i-1];
				exl=in->ex[i-1];
				phil=in->phi[i-1];
				yl=in->ymesh[i-1];
				Tll=in->Tl[i-1];
				Tel=in->Te[i-1];
				Thl=in->Th[i-1];
				Ecl=in->Ec[i-1];
				Evl=in->Ev[i-1];

				for (band=0;band<bands;band++)
				{
					mel=in->me[i-1];
					mhl=in->mh[i-1];

					nl[band]=in->n[i-1][band];
					dnl[band]=in->dn[i-1][band];

					dnldTe[band]=get_dndT_den(in->x[i-1][band]+in->t[i-1][band],Tel,mel,in->dostype[i],dos_all);
					wnl[band]=get_n_w(in->x[i-1][band]+in->t[i-1][band],Tel,mel,in->dostype[i]);

					dpldTh[band]=get_dpdT_den(in->xp[i-1][band]-in->tp[i-1][band],Thl,mhl,in->dostype[i],dos_all);
					wpl[band]=get_p_w(in->xp[i-1][band]-in->tp[i-1][band],Thl,mhl,in->dostype[i]);

					pl[band]=in->p[i-1][band];
					dpl[band]=in->dp[i-1][band];
					munl[band]=in->mun[i-1][band];
					mupl[band]=in->mup[i-1][band];
				}

				epl=in->epsilonr[i-1]*epsilon0;


				kll=in->kl[i-1];
			}

			Ecc=-in->Xi[i]-in->phi[i];
			Evc=-in->Xi[i]-in->phi[i]-in->Eg[i];

			if (i==(in->ymeshpoints-1))
			{

				Dexr=in->Dex[i];
				exr=0.0;
				phir=in->Vr;
				yr=in->ymesh[i]+(in->ymesh[i]-in->ymesh[i-1]);
				Tlr=in->Tlr;
				Ter=in->Tlr;
				Thr=in->Tlr;

				Ecr=-in->Xi[i]-phir;
				Evr=-in->Xi[i]-phir-in->Eg[i];

				mer=in->me[i];
				mhr=in->mh[i];

				for (band=0;band<bands;band++)
				{
					xnr=phir+in->Fi[i];
					tnr=in->Xi[i];
					if (forcecharge==TRUE)
					{
						one=in->deltaFrn;
					}else
					{
						one=xnr+tnr;
					}
					nr[band]=get_n_den(one,Ter,mer,in->bdos,dos_all);
					dnr[band]=get_dn_den(one,Ter,mer,in->bdos,dos_all);
					//printf("rod = %e\n",nr);
					wnr[band]=get_n_w(one,Ter,mer,in->bdos);
					dnrdTe[band]=0.0;


					xpr=-(phir+in->Fi[i]);
					tpr=in->Xi[i]+in->Eg[i];
					if (forcecharge==TRUE)
					{
						one=in->deltaFrp;
					}else
					{
						one=xpr-tpr;
					}
					pr[band]=get_p_den(one,Thr,mhr,in->bdos,dos_all);
					dpr[band]=get_dp_den(one,Thr,mhr,in->bdos,dos_all);
					wpr[band]=get_p_w(one,Thr,mhr,in->bdos);
					dprdTh[band]=0.0;
					munr[band]=in->mun[i][band];
					mupr[band]=in->mup[i][band];
				}
				epr=in->epsilonr[i]*epsilon0;
				klr=in->kl[i];

			}else
			{
				mer=in->me[i+1];
				mhr=in->mh[i+1];

				Dexr=in->Dex[i+1];
				exr=in->ex[i+1];
				phir=in->phi[i+1];
				yr=in->ymesh[i+1];
				Tlr=in->Tl[i+1];
				Ter=in->Te[i+1];
				Thr=in->Th[i+1];

				Ecr=in->Ec[i+1];
				Evr=in->Ev[i+1];

				for (band=0;band<bands;band++)
				{
					nr[band]=in->n[i+1][band];
					dnr[band]=in->dn[i+1][band];
					dnrdTe[band]=get_dndT_den(in->x[i+1][band]+in->t[i+1][band],in->Te[i+1],mer,in->dostype[i],dos_all);
					wnr[band]=get_n_w(in->x[i+1][band]+in->t[i+1][band],in->Te[i+1],mer,in->dostype[i]);

					dprdTh[band]=get_dpdT_den(in->xp[i+1][band]-in->tp[i+1][band],in->Th[i+1],mhr,in->dostype[i],dos_all);
					wpr[band]=get_p_w(in->xp[i+1][band]-in->tp[i+1][band],in->Th[i+1],mhr,in->dostype[i]);

					pr[band]=in->p[i+1][band];
					dpr[band]=in->dp[i+1][band];
					munr[band]=in->mun[i+1][band];
					mupr[band]=in->mup[i+1][band];
				}
				epr=in->epsilonr[i+1]*epsilon0;
				klr=in->kl[i+1];

			}


			tauncapc=in->tauncap[i];
			taunescapec=in->taunescape[i];

			taupcapc=in->taupcap[i];
			taupescapec=in->taupescape[i];

			mec=in->me[i];
			mhc=in->mh[i];

			epc=in->epsilonr[i]*epsilon0;


			exc=in->ex[i];
			Dexc=in->Dex[i];
			yc=in->ymesh[i];
			dyl=yc-yl;
			dyr=yr-yc;
			ddh=(dyl+dyr)/2.0;
			dh=(dyl+dyr);
			phic=in->phi[i];
			Tlc=in->Tl[i];
			Tec=in->Te[i];
			Thc=in->Th[i];
			ni=in->ni[i];

			for (band=0;band<bands;band++)
			{

				munc[band]=in->mun[i][band];
				mupc[band]=in->mup[i][band];


				//mupl[band]=(mupl[band]+mupr[band])/2.0;
				//mupc[band]=mupl[band];
				//mupr[band]=mupl[band];//mupl[band];

				//munl[band]=1e-7;//(munl[band]+munr[band])/2.0;
				//munc[band]=1e-7;
				//munr[band]=1e-7;//munl[band];

				dncdTe[band]=get_dndT_den(in->x[i][band]+in->t[i][band],in->Te[i],mec,in->dostype[i],dos_all);
				dpcdTh[band]=get_dpdT_den(in->xp[i][band]-in->tp[i][band],in->Th[i],mhc,in->dostype[i],dos_all);
				wnc[band]=get_n_w(in->x[i][band]+in->t[i][band],in->Te[i],mec,in->dostype[i]);
				wpc[band]=get_p_w(in->xp[i][band]-in->tp[i][band],in->Th[i],mhc,in->dostype[i]);

				Dnl[band]=munl[band]*(2.0/3.0)*wnl[band]/q;
				Dpl[band]=mupl[band]*(2.0/3.0)*wpl[band]/q;

				Dnc[band]=munc[band]*(2.0/3.0)*wnc[band]/q;
				Dpc[band]=mupc[band]*(2.0/3.0)*wpc[band]/q;

				Dnr[band]=munr[band]*(2.0/3.0)*wnr[band]/q;
				Dpr[band]=mupr[band]*(2.0/3.0)*wpr[band]/q;
fprintf(file_j,"%e %e %e\n",Dnl[band],Dnc[band],Dnr[band]);

				Dnl[band]=(Dnl[band]+Dnc[band])/2.0;
				Dnr[band]=(Dnr[band]+Dnc[band])/2.0;

				Dpl[band]=(Dpl[band]+Dpc[band])/2.0;
				Dpr[band]=(Dpr[band]+Dpc[band])/2.0;

				munl[band]=(munl[band]+munc[band])/2.0;
				munr[band]=(munr[band]+munc[band])/2.0;

				mupl[band]=(mupl[band]+mupc[band])/2.0;
				mupr[band]=(mupr[band]+mupc[band])/2.0;



				nc[band]=in->n[i][band];
				pc[band]=in->p[i][band];

				dnc[band]=in->dn[i][band];
				dpc[band]=in->dp[i][band];

				nc_all[band]=get_n_den(in->x[i][band]+in->t[i][band],in->Te[i],in->me[i],in->dostype[i],dos_all);
				pc_all[band]=get_p_den(in->xp[i][band]-in->tp[i][band],in->Th[i],in->mh[i],in->dostype[i],dos_all);
				dnc_all[band]=get_dn_den(in->x[i][band]+in->t[i][band],in->Te[i],in->me[i],in->dostype[i],dos_all);
				dpc_all[band]=get_dp_den(in->xp[i][band]-in->tp[i][band],in->Th[i],in->mh[i],in->dostype[i],dos_all);



			}


			//ntc=get_n_trap_den(in->xt[i]+in->tt[i],in->Te[i],in->me[i],in->dostype[i],0);
			//dntc=get_dn_trap_den(in->xt[i]+in->tt[i],in->Te[i],in->me[i],in->dostype[i],0);

			//ptc=get_p_trap_den(in->xpt[i]-in->tpt[i],in->Th[i],in->mh[i],in->dostype[i],0);
			//dptc=get_dp_trap_den(in->xpt[i]-in->tpt[i],in->Th[i],in->mh[i],in->dostype[i],0);



			if (in->ntrapnewton)
			{
				ntc=get_n_trap_den(in->xt[i]+in->tt[i],in->Te[i],in->me[i],in->dostype[i],0);
				dntc=get_dn_trap_den(in->xt[i]+in->tt[i],in->Te[i],in->me[i],in->dostype[i],0);
			}else if (in->combitrap)
			{
				ntc=get_n_trap_den(in->x[i][0]+in->t[i][0],in->Te[i],in->me[i],in->dostype[i],0);
				dntc=get_dn_trap_den(in->x[i][0]+in->t[i][0],in->Te[i],in->me[i],in->dostype[i],0);
			}else
			{
				ntc=0.0;
				dntc=0.0;
			}

			//printf("%e %e\n",nc[0],ntc);
			//getchar();

			if (in->ptrapnewton)
			{
				ptc=get_p_trap_den(in->xpt[i]-in->tpt[i],in->Th[i],in->mh[i],in->dostype[i],0);
				dptc=get_dp_trap_den(in->xpt[i]-in->tpt[i],in->Th[i],in->mh[i],in->dostype[i],0);
			}else if (in->combitrap)
			{
				ptc=get_p_trap_den(in->xp[i][0]-in->tp[i][0],in->Th[i],in->mh[i],in->dostype[i],0);
				dptc=get_dp_trap_den(in->xp[i][0]-in->tp[i][0],in->Th[i],in->mh[i],in->dostype[i],0);
			}
			else
			{
				ptc=0.0;
				dptc=0.0;
			}



			klc=in->kl[i];
			nlast=in->nlast[i];
			plast=in->plast[i];
			dt=in->dt;

	R=in->R[i];
	Gex=in->Gex[i];

	e0=(epl+epc)/2.0;
	e1=(epc+epr)/2.0;

	kl0=(klc+kll)/2.0;
	kl1=(klr+klc)/2.0;

	dphil=-e0/dyl/ddh;
	dphic=e0/dyl/ddh+e1/dyr/ddh;
	dphir=-e1/dyr/ddh;
	Rbi=0.0;
	deriv=phil*dphil+phic*dphic+phir*dphir;

	for (band=0;band<bands;band++)
	{
		dphidxic[band]=q*(dnc[band]);
		dphidxipc[band]=-q*(dpc[band]);

		if (in->ntrapnewton==TRUE)
		{
			dphidxic[band]+=q*(in->dnt[i]);
			dphidntrap=q*(dntc);
		}

		if (in->ptrapnewton==TRUE)
		{
			dphidptrap=-q*(dptc);
			dphidxipc[band]+=-q*(in->dpt[i]);
		}

		//if (in->combitrap)
		//{
		//	dphidxic[band]+=q*(in->dnt[i]);
		//	dphidxipc[band]+=-q*(in->dpt[i]);
		//}
	}



	//double kdiss=0.0;
	double mu_min=0.0;

	double langevin=0.0;

	if (in->Rbitype==0)
	{
		mu_min=in->mun_avg[i];
		if (mu_min>in->mup_avg[i]) mu_min=in->mup_avg[i];
		langevin=q*(mu_min)/(epc);
		Rbi_mul=1.0;

	}else
	if (in->Rbitype==1)
	{
		mu_min=(in->mup_avg[i]+in->mun_avg[i])/2.0;
		langevin=q*(mu_min)/(epc);
		//printf("%e\n",langevin);
		Rbi_mul=in->alpha;

	}else
	if (in->Rbitype==2)
	{
		mu_min=(in->mup_avg[i]+in->mun_avg[i])/2.0;
		langevin=q*(mu_min)/(epc);
		//printf("%e\n",langevin);
		double rc=q*q/(4.0*pi*epsilon0*in->epsilonr[i]*kb*Tlc);
		double A=in->A[i];
		double deltaE=in->deltaE[i];
		//printf("%e %e\n",deltaE,A);
		double R=in->a[i];
		double Dp=A*exp(-(deltaE*q/(kb*Tlc)));
		Rbi_mul=1.0/(1-exp(-rc/R)+(Dp*rc/R/R)*exp(-rc/R));
		//printf("%e %e\n",q*(mu_min)*exp(-q*0.178/(kb*Tlc))/(epc),Rbi_mul*langevin);
	}
	else
	if (in->Rbitype==3)
	{
		Rbi_mul=1.0;
		mu_min=(in->mup_avg[i]+in->mun_avg[i])/2.0;
		langevin=q*(mu_min)*exp(-q*in->deltaE[i]/(kb*Tlc))/(epc);
	}else
	if (in->Rbitype==4)
	{
		Rbi_mul=1.0;
		langevin=get_mat_param_xy(&(in->mat.l[in->ymatlayers[i]]),mat_rbi,sqrt(fabs(nc[0]*pc[0]-ni*ni)));
		//printf("Here I am %e\n",langevin);
		if (langevin<0) langevin=1e-30;
	}else
	if (in->Rbitype==5)
	{
		//printf("Calculating with this\n");
		Rbi_mul=deibel;
		//printf("%e\n",deibel);
		langevin=(in->mup_avg[i]+in->mun_avg[i])/2.0;
	}else
	if (in->Rbitype==6)
	{
		//printf("Calculating with this\n");
		Rbi_mul=1.0;
		//printf("%e\n",deibel);
		langevin=q*2.0*sqrt(in->mup_avg[i]*in->mun_avg[i])/epc;
	}

	//printf("%e %e %e\n",langevin,nc*pc-ni*ni,sqrt(fabs(nc*pc-ni*ni)));
	in->Rbi_k[i]=Rbi_mul*langevin;
	in->Rbi[i]=Rbi_mul*langevin*(nc[0]*pc[0]);

	Rbi=in->Rbi[i];
	if (Rbi<=0.0)
	{
		Rbi=0.0;
		Rbi_mul=0.0;
	}

	//printf("%e\n",in->Rbi[i]);
	//printf("%e\n",Rbi);
	//double dexdn=0.0;
	//double dexdp=0.0;

	//double dJndex=0.0;
	//double dJpdex=0.0;

	//double kf=in->kf[i];
	double dRbidxic=Rbi_mul*langevin*(dnc[0]*pc[0]);
	double dRbidxipc=Rbi_mul*langevin*(nc[0]*dpc[0]);
	//double Rdis=0.0;

	/*if (in->enable_ex==TRUE)
	{
		kdiss=in->kd[i];
		Rdis=kdiss*exc;
		in->G[i]=Rdis;
		dexdn=0.0;
		dexdp=0.0;Dexc=0.0;
		dexl=Dexc/dyl/(dyl+dyr)/2.0;
		dexc=-Dexc*(1.0/dyl+1.0/dyr)/(dyl+dyr)/2.0;
		dexr=Dexc/dyr/(dyl+dyr)/2.0;

		dex=dexl*exl+dexc*exc+dexr*exr +Gex-kdiss*exc-kf*exc+Rbi;

		G=in->G[i];


		dexc+=-kdiss-kf;
	}*/

	if (in->Rbi_on==FALSE) Rbi=0.0;

G=in->G[i];

			for (band=0;band<bands;band++)
			{

xil[band]=q*2.0*(3.0/2.0)*(Ecc-Ecl)/((wnc[band]+wnl[band]))-2.0*(3.0/2.0)*(mec-mel)/(mec+mel)+2.0*(wnc[band]-wnl[band])/(wnc[band]+wnl[band]);
xir[band]=q*2.0*(3.0/2.0)*(Ecr-Ecc)/((wnr[band]+wnc[band]))-2.0*(3.0/2.0)*(mer-mec)/(mer+mec)+2.0*(wnr[band]-wnc[band])/(wnr[band]+wnc[band]);

xipl[band]=q*2.0*(3.0/2.0)*(Evc-Evl)/((wpc[band]+wpl[band]))-2.0*(3.0/2.0)*(mhc-mhl)/(mhc+mhl)-2.0*(wpc[band]-wpl[band])/(wpc[band]+wpl[band]);
xipr[band]=q*2.0*(3.0/2.0)*(Evr-Evc)/((wpr[band]+wpc[band]))-2.0*(3.0/2.0)*(mhr-mhc)/(mhr+mhc)-2.0*(wpr[band]-wpc[band])/(wpr[band]+wpc[band]);


			Jnl[band]=(Dnl[band]/dyl)*(B(-xil[band])*nc[band]-B(xil[band])*nl[band]);
			Jnr[band]=(Dnr[band]/dyr)*(B(-xir[band])*nr[band]-B(xir[band])*nc[band]);


			Jpl[band]=(Dpl[band]/dyl)*(B(-xipl[band])*pl[band]-B(xipl[band])*pc[band]);
			Jpr[band]=(Dpr[band]/dyr)*(B(-xipr[band])*pc[band]-B(xipr[band])*pr[band]);


			in->Jn[i][band]=q*(Jnl[band]+Jnr[band])/2.0;
			in->Jp[i][band]=q*(Jpl[band]+Jpr[band])/2.0;

			dJdxil[band]=(Dnl[band]/dyl/ddh)*(B(xil[band])*dnl[band]);
			dJdxic[band]=-(Dnl[band]/dyl)*B(-xil[band])*dnc[band]/ddh-(Dnr[band]/dyr)*B(xir[band])*dnc[band]/ddh;
			dJdxir[band]=(Dnr[band]/dyr)*B(-xir[band])*dnr[band]/ddh;


			dJpdxipl[band]=-(Dpl[band]/dyl/ddh)*(B(-xipl[band])*dpl[band]);
			dJpdxipc[band]=((Dpl[band]/dyl/ddh)*B(xipl[band])*dpc[band]+(Dpr[band]/dyr/ddh)*B(-xipr[band])*dpc[band]);
			dJpdxipr[band]=-(Dpr[band]/dyr/ddh)*B(xipr[band])*dpr[band];

			dJdphil[band]=(munl[band]/dyl/ddh)*(dB(-xil[band])*nc[band]+dB(xil[band])*nl[band]);
			dJdphic[band]=-((munl[band]/dyl)*(dB(-xil[band])*nc[band]+dB(xil[band])*nl[band])/ddh+(munr[band]/dyr)*(dB(-xir[band])*nr[band]+dB(xir[band])*nc[band])/ddh);
			dJdphir[band]=(munr[band]/dyr/ddh)*(dB(-xir[band])*nr[band]+dB(xir[band])*nc[band]);


			dJpdphil[band]=((mupl[band])/dyl/ddh)*(dB(-xipl[band])*pl[band]+dB(xipl[band])*pc[band]);
			dJpdphic[band]=-(((mupl[band])/dyl/ddh)*(dB(-xipl[band])*pl[band]+dB(xipl[band])*pc[band])+(mupr[band]/dyr/ddh)*(dB(-xipr[band])*pc[band]+dB(xipr[band])*pr[band]));
			dJpdphir[band]=(mupr[band]/dyr/ddh)*(dB(-xipr[band])*pc[band]+dB(xipr[band])*pr[band]);


			}


			for (band=0;band<bands;band++)
			{
				if (in->go_time==TRUE)
				{
					dJdxic[band]+=-dnc[band]/dt;
				}

				if (in->go_time==TRUE)
				{
					dJpdxipc[band]+=dpc[band]/dt;
				}

			}

			dJdxipc=0.0;
			if (in->Rbi_on==TRUE)
			{
				dJdxic[0]+=-dRbidxic;
				dJdxipc+=-dRbidxipc;
			}




			dJpdxic=0.0;
			if (in->Rbi_on==TRUE)
			{
				dJpdxipc[0]+=dRbidxipc;
				dJpdxic+=dRbidxic;
			}


			//Electron thermal

/*if (in->nnewton==TRUE)
{
	double sxil;
	double sxir;
	if (i==0) Ecl=Ecc;
	//////////////////Calculate xi////////////////////////////////
	sxil=q*2.0*(Ecc-Ecl)/(kb*(Tec+Tel))-2.0*(3.0/2.0)*(mec-mel)/(mec+mel)+2.0*(Tec-Tel)/(Tec+Tel);
	sxir=q*2.0*(Ecr-Ecc)/(kb*(Ter+Tec))-2.0*(3.0/2.0)*(mer-mec)/(mer+mec)+2.0*(Ter-Tec)/(Ter+Tec);

	double dsxildTel=-q*2.0*(Ecc-Ecl)/(kb*pow((Tec+Tel),2.0))-2.0*(Tec-Tel)/pow((Tec+Tel),2.0)-2.0/(Tec+Tel);
	double dsxildTec=-q*2.0*(Ecc-Ecl)/(kb*pow((Tec+Tel),2.0))-2.0*(Tec-Tel)/pow((Tec+Tel),2.0)+2.0/(Tec+Tel);

	double dsxirdTec=-q*2.0*(Ecr-Ecc)/(kb*pow((Ter+Tec),2.0))-2.0*(Ter-Tec)/(pow((Ter+Tec),2.0))-2.0/(Ter+Tec);
	double dsxirdTer=-q*2.0*(Ecr-Ecc)/(kb*pow((Ter+Tec),2.0))-2.0*(Ter-Tec)/(pow((Ter+Tec),2.0))+2.0/(Ter+Tec);

	double dsxildphil=q*2.0/(kb*(Tec+Tel));
	double dsxildphic=-q*2.0/(kb*(Tec+Tel));
	double dsxirdphic=q*2.0/(kb*(Ter+Tec));
	double dsxirdphir=-q*2.0/(kb*(Ter+Tec));

	///////////////////////Calculate fluxes/////////////////////////
	Snl=0.0;
	Snr=0.0;
	Snl_pre=(-(5.0/2.0)*munl*pow((kb*(Tec+Tel)/2.0),2.0)/(q*dyl));
	Snr_pre=(-(5.0/2.0)*munr*pow((kb*(Ter+Tec)/2.0),2.0)/(q*dyr));
	dSnl_predTe=-(5.0/2.0)*munl*kb*kb*(Tec+Tel)/2.0/(q*dyl);
	dSnr_predTe=-(5.0/2.0)*munr*kb*kb*(Ter+Tec)/2.0/(q*dyr);


	Snl=Snl_pre*(B(-sxil)*nc-B(sxil)*nl);
	Snr=Snr_pre*(B(-sxir)*nr-B(sxir)*nc);
					/////Calculate dJndTe////////
					//printf("%e %e %e ",dnldTe,dncdTe,dnrdTe);
					//getchar();
					//dJedTe

	dxildTel=-q*2.0*(Ecc-Ecl)/(kb*pow((Tec+Tel),2.0))-2.0/(Tec+Tel)-2.0*(Tec-Tel)/pow((Tec+Tel),2.0);
	dxildTec=-q*2.0*(Ecc-Ecl)/(kb*pow((Tec+Tel),2.0))+2.0/(Tec+Tel)-2.0*(Tec-Tel)/pow((Tec+Tel),2.0);

	dxirdTec=-q*2.0*(Ecr-Ecc)/(kb*pow((Ter+Tec),2.0))-2.0/(Ter+Tec)-2.0*(Ter-Tec)/pow((Ter+Tec),2.0);
	dxirdTer=-q*2.0*(Ecr-Ecc)/(kb*pow((Ter+Tec),2.0))+2.0/(Ter+Tec)-2.0*(Ter-Tec)/pow((Ter+Tec),2.0);

		//		Jnl=(Dnl/dyl)*(B(-xil)*nc-B(xil)*nl);
		//		Jnr=(Dnr/dyr)*(B(-xir)*nr-B(xir)*nc);

	///////////////dJndTe//////////////////////////////////////
	dJnldTe=0.0;//((munl*((Tel+Tec)/2.0)*(kb/q))/dyl/ddh)*(B(xil)*dnldTe);
	dJncdTe=0.0;//-((munl*((Tec+Tel)/2.0)*(kb/q))/dyl)*B(-xil)*dncdTe/ddh-((munr*((Ter+Tec)/2.0)*(kb/q))/dyr)*B(xir)*dncdTe/ddh;
	dJnrdTe=0.0;//((munr*((Ter+Tec)/2.0)*(kb/q))/dyr/ddh)*B(-xir)*dnrdTe;

	dJnldTe+=(Dnl/dyl/ddh)*dB(-xil)*nc*dxildTel+(Dnl/dyl/ddh)*dB(xil)*nl*dxildTel;
	dJncdTe+=(Dnl/dyl/ddh)*dB(-xil)*dxildTec*nc+(Dnl/dyl/ddh)*dB(xil)*dxildTec*nl-(Dnr/dyr/ddh)*dB(-xir)*dxirdTec*nr-(Dnr/dyr/ddh)*dxirdTec*dB(xir)*nc;
	dJnrdTe+=-(Dnr/dyr/ddh)*dB(-xir)*dxirdTer*nr-(Dnr/dyr/ddh)*dB(xir)*dxirdTer*nc;


	dJnldTe+=-(munl*(1.0/2.0)*(kb/q)/dyl/ddh)*B(-xil)*nc+(munl*(1.0/2.0)*(kb/q)/dyl/ddh)*B(xil)*nl;
	dJncdTe+=-(munl*(1.0/2.0)*(kb/q)/dyl/ddh)*B(-xil)*nc+(munl*(1.0/2.0)*(kb/q)/dyl/ddh)*B(xil)*nl+(munr*(1.0/2.0)*(kb/q)/dyr/ddh)*B(-xir)*nr-(munr*(1.0/2.0)*(kb/q)/dyr/ddh)*B(xir)*nc;
	dJnrdTe+=(munr*(1.0/2.0)*(kb/q)/dyr/ddh)*B(-xir)*nr-(munr*(1.0/2.0)*(kb/q)/dyr/ddh)*B(xir)*nc;


	if (in->go_time==TRUE)
	{
		//dJncdTe+=-dncdTe/dt;
		//printf("%e\n",dncdTe/dt);
		//getchar();
	}
	//	dJncdTe+=-langevin*(dncdTe*pc);


					//dJnldTe=0.0;
					//dJncdTe=0.0;
					//dJnrdTe=0.0;



	////////////////////Calculate dphidTe//////////////////
						dphidTe=q*(dncdTe);

	///////////////////////Calculate dTeldphi//////////////////////////////
	dTeldphi=Snl_pre*(-dB(-sxil)*dsxildphil*nc-dB(sxil)*dsxildphil*nl)/ddh;
	dTecdphi=Snl_pre*(-dB(-sxil)*dsxildphic*nc-dB(sxil)*dsxildphic*nl)/ddh-Snr_pre*(-dB(-sxir)*dsxirdphic*nr-dB(sxir)*dsxirdphic*nc)/ddh;
	dTerdphi=-Snr_pre*(-dB(-sxir)*dsxirdphir*nr-dB(sxir)*dsxirdphir*nc)/ddh;

	if (in->jouleheat==TRUE)
	{
		dTeldphi+=q*(Jnr+Jnl)/2.0/dh;
		dTerdphi+=-q*(Jnr+Jnl)/2.0/dh;
	}
	///////////////////////////dTe///////////////////////////////
	dTe=(Snl-Snr)/ddh-(3.0/2.0)*kb*nc*(Tec-Tlc)/in->tau_e;//+in->Hasorb[i]/2.0;

	if (in->jouleheat==TRUE)
	{
		dTe+=q*(Ecr-Ecl)*(Jnr+Jnl)/2.0/dh;
		//printf("%e\n",q*(Evr-Evl)*(Jpr+Jpl)/2.0/ddh);
		//getchar();
	}



	dTel=Snl_pre*(-dB(-sxil)*dsxildTel*nc-dB(sxil)*dsxildTel*nl)/ddh;
	dTec=Snl_pre*(-dB(-sxil)*dsxildTec*nc-dB(sxil)*dsxildTec*nl)/ddh-Snr_pre*(-dB(-sxir)*dsxirdTec*nr-dB(sxir)*dsxirdTec*nc)/ddh;
	dTer=-Snr_pre*(-dB(-sxir)*dsxirdTer*nr-dB(sxir)*dsxirdTer*nc)/ddh;

	dTel+=dSnl_predTe*(B(-sxil)*nc-B(sxil)*nl)/ddh;
	dTec+=dSnl_predTe*(B(-sxil)*nc-B(sxil)*nl)/ddh-dSnr_predTe*(B(-sxir)*nr-B(sxir)*nc)/ddh;
	dTer+=-dSnr_predTe*(B(-sxir)*nr-B(sxir)*nc)/ddh;

	//dTel+=-Snl_pre*B(sxil)*dnldTe/ddh;
	//dTec+=Snl_pre*B(-sxil)*dncdTe/ddh+Snr_pre*B(sxir)*dncdTe/ddh;
	//dTer+=-Snr_pre*B(-sxir)*dnrdTe/ddh;

	if (in->jouleheat==TRUE)
	{
		//Jnl=(Dnl/dyl)*(B(-xil)*nc-B(xil)*nl);
		//Jnr=(Dnr/dyr)*(B(-xir)*nr-B(xir)*nc);
		//dTel+=q*(Ecr-Ecl)*(Dnl/dyl/dh/2.0)*(-dB(-xil)*dxildTel*nc-dB(xil)*dxildTel*nl);
		//dTec+=q*(Ecr-Ecl)*(Dnl/dyl/dh/2.0)*(-dB(-xil)*dxildTec*nc-dB(xil)*dxildTec*nl)+q*(Ecr-Ecl)*(Dnr/dyr/dh/2.0)*(-dB(-xir)*dxirdTec*nr-dB(xir)*dxirdTec*nc);
		//dTer+=q*(Ecr-Ecl)*(Dnr/dyr/dh/2.0)*(-dB(-xir)*dxirdTer*nr-dB(xir)*dxirdTer*nc);


		dTel+=q*(Ecr-Ecl)*(munl*(1.0/2.0)*(kb/q)/dyl/dh/2.0)*(B(-xil)*dxildTel*nc-B(xil)*dxildTel*nl);
		dTec+=q*(Ecr-Ecl)*(munl*(1.0/2.0)*(kb/q)/dyl/dh/2.0)*(B(-xil)*dxildTec*nc-B(xil)*dxildTec*nl)+q*(Ecr-Ecl)*(munr*(1.0/2.0)*(kb/q)/dyr/dh/2.0)*(B(-xir)*dxirdTec*nr-B(xir)*dxirdTec*nc);
		dTer+=q*(Ecr-Ecl)*(munr*(1.0/2.0)*(kb/q)/dyr/dh/2.0)*(B(-xir)*dxirdTer*nr-B(xir)*dxirdTer*nc);
	}

	dTec+=-(3.0/2.0)*kb*nc/in->tau_e;
	//dTec+=-(3.0/2.0)*kb*Tec*dncdTe/in->tau_e;





	/////Calculate dTedn////////
	dTeldn=-Snl_pre*B(sxil)*dnl/ddh;
	dTecdn=Snl_pre*B(-sxil)*dnc/ddh+Snr_pre*B(sxir)*dnc/ddh;
	dTerdn=-Snr_pre*B(-sxir)*dnr/ddh;

	if (in->jouleheat==TRUE)
	{
	dTeldn+=-q*(Ecr-Ecl)*(Dnl/dyl/dh/2.0)*B(xil)*dnl;
	dTecdn+=q*(Ecr-Ecl)*(Dnl/dyl/dh/2.0)*B(-xil)*dnc-q*(Ecr-Ecl)*(Dnr/dyr/dh/2.0)*B(xir)*dnc;
	dTerdn+=q*(Ecr-Ecl)*(Dnr/dyr/dh/2.0)*B(-xir)*dnr;
	}
	//Jnl=(Dnl/dyl)*(B(-xil)*nc-B(xil)*nl);
	//Jnr=(Dnr/dyr)*(B(-xir)*nr-B(xir)*nc);

	dTecdn+=-(3.0/2.0)*kb*dnc*(Tec-Tlc)/in->tau_e;
}*/


/*if (in->hnewton==TRUE)
{
	if (i==0) Evl=Evc;


	//-2.0*(3.0/2.0)*(mhc-mhl)/(mhc+mhl)
	//-2.0*(3.0/2.0)*(mhr-mhc)/(mhr+mhc)
	sxipl=q*2.0*(Evc-Evl)/(kb*(Thc+Thl))-2.0*(Thc-Thl)/(Thc+Thl);
	sxipr=q*2.0*(Evr-Evc)/(kb*(Thr+Thc))-2.0*(Thr-Thc)/(Thr+Thc);



	dsxilpdThl=-q*2.0*(Evc-Evl)/(kb*pow((Thc+Thl),2.0))+2.0/(Thc+Thl)+2.0*(Thc-Thl)/pow((Thc+Thl),2.0);
	dsxilpdThc=-q*2.0*(Evc-Evl)/(kb*pow((Thc+Thl),2.0))-2.0/(Thc+Thl)+2.0*(Thc-Thl)/pow((Thc+Thl),2.0);

	dsxirpdThc=-q*2.0*(Evr-Evc)/(kb*pow((Thr+Thc),2.0))+2.0/(Thr+Thc)+2.0*(Thr-Thc)/pow((Thr+Thc),2.0);
	dsxirpdThr=-q*2.0*(Evr-Evc)/(kb*pow((Thr+Thc),2.0))-2.0/(Thr+Thc)+2.0*(Thr-Thc)/pow((Thr+Thc),2.0);


//xipl=q*2.0*(Evc-Evl)/(kb*(Thc+Thl))-2.0*(3.0/2.0)*(mhc-mhl)/(mhc+mhl)-2.0*(Thc-Thl)/(Thc+Thl);
//xipr=q*2.0*(Evr-Evc)/(kb*(Thr+Thc))-2.0*(3.0/2.0)*(mhr-mhc)/(mhr+mhc)-2.0*(Thr-Thc)/(Thr+Thc);


	dxilpdThl=-q*2.0*(Evc-Evl)/(kb*pow((Thc+Thl),2.0))+2.0/(Thc+Thl)+2.0*(Thc-Thl)/pow((Thc+Thl),2.0);
	dxilpdThc=-q*2.0*(Evc-Evl)/(kb*pow((Thc+Thl),2.0))-2.0/(Thc+Thl)+2.0*(Thc-Thl)/pow((Thc+Thl),2.0);
	dxirpdThc=-q*2.0*(Evr-Evc)/(kb*pow((Thr+Thc),2.0))+2.0/(Thr+Thc)+2.0*(Thr-Thc)/pow((Thr+Thc),2.0);
	dxirpdThr=-q*2.0*(Evr-Evc)/(kb*pow((Thr+Thc),2.0))-2.0/(Thr+Thc)+2.0*(Thr-Thc)/pow((Thr+Thc),2.0);


	Spl=0.0;
	Spr=0.0;
	Spl_pre=((5.0/2.0)*mupl*pow((kb*(Thc+Thl)/2.0),2.0)/(q*dyl));
	Spr_pre=((5.0/2.0)*mupr*pow((kb*(Thr+Thc)/2.0),2.0)/(q*dyr));
	dSpl_predTh=(5.0/2.0)*mupl*kb*kb*(Thc+Thl)/2.0/(q*dyl);
	dSpr_predTh=(5.0/2.0)*mupr*kb*kb*(Thr+Thc)/2.0/(q*dyr);
	//printf("%e %e %e\n",mupr,Thr,Thc);

	Spl=Spl_pre*(B(-sxipl)*pl-B(sxipl)*pc);
	Spr=Spr_pre*(B(-sxipr)*pc-B(sxipr)*pr);

	dThl=0.0;
	dThc=0.0;
	dThr=0.0;

	dTh=(Spl-Spr)/ddh-(3.0/2.0)*kb*pc*(Thc-Tlc)/in->tau_h;//+in->Hasorb[i]/2.0;

	if (in->jouleheat==TRUE)
	{
		dTh+=q*(Evr-Evl)*(Jpr+Jpl)/2.0/dh;

		//printf("%e\n",q*(Evr-Evl)*(Jpr+Jpl)/2.0/ddh);
		//getchar();
	}
					//printf("%e",in->Hasorb[i]/2.0);

	///////////////////////dThdTh///////////////////////////////////////////

	//dThl=Spl_pre*(-dB(-sxipl)*dsxilpdThl*pl-dB(sxipl)*dsxilpdThl*pc)/ddh;
	//dThc=Spl_pre*(-dB(-sxipl)*dsxilpdThc*pl-dB(sxipl)*dsxilpdThc*pc)/ddh-Spr_pre*(-dB(-sxipr)*dsxirpdThc*pc-B(sxipr)*dsxirpdThc*pr)/ddh;
	//dThr=-Spr_pre*(-dB(-sxipr)*dsxirpdThr*pc-B(sxipr)*dsxirpdThr*pr)/ddh;

	dThl+=dSpl_predTh*(B(-sxipl)*pl-B(sxipl)*pc)/ddh;
	dThc+=dSpl_predTh*(B(-sxipl)*pl-B(sxipl)*pc)/ddh-dSpr_predTh*(B(-sxipr)*pc-B(sxipr)*pr)/ddh;
	dThr+=-dSpr_predTh*(B(-sxipr)*pc-B(sxipr)*pr)/ddh;

	dThc+=-(3.0/2.0)*kb*pc/in->tau_h;


	///////////////////////dJhdTh/////////////////////////////////////////////
	dJpldTh=0.0;
	dJpcdTh=0.0;
	dJprdTh=0.0;

	//Jpl=(Dpl/dyl)*(B(-xipl)*pl-B(xipl)*pc);
	//Jpr=(Dpr/dyr)*(B(-xipr)*pc-B(xipr)*pr);


	dJpldTh+=-((mupl*(1.0/2.0)*(kb/q)/dyl/ddh)*B(-xipl)*pl-(mupl*(1.0/2.0)*(kb/q)/dyl/ddh)*B(xipl)*pc);
	dJpcdTh+=-((mupl*(1.0/2.0)*(kb/q)/dyl/ddh)*B(-xipl)*pl-(mupl*(1.0/2.0)*(kb/q)/dyl/ddh)*B(xipl)*pc)+((mupr*(1.0/2.0)*(kb/q)/dyr/ddh)*B(-xipr)*pc-(mupr*(1.0/2.0)*(kb/q)/dyr/ddh)*B(xipr)*pr);
	dJprdTh+=(mupr*(1.0/2.0)*(kb/q)/dyr/ddh)*B(-xipr)*pc-(mupr*(1.0/2.0)*(kb/q)/dyr/ddh)*B(xipr)*pr;

	//I have taken this out because it causes instability but it should be in there.
	//dJpldTh+=-((mupl*((Thc+Thl)/2.0)*(kb/q)/dyl/ddh)*-dB(-xipl)*dxilpdThl*pl-(mupl*((Thc+Thl)/2.0)*(kb/q)/dyl/ddh)*dB(xipl)*dxilpdThl*pc);
	//dJpcdTh+=-((mupl*((Thc+Thl)/2.0)*(kb/q)/dyl/ddh)*-dB(-xipl)*dxilpdThc*pl-(mupl*((Thc+Thl)/2.0)*(kb/q)/dyl/ddh)*dB(xipl)*dxilpdThc*pc)-((mupr*((Thr+Thc)/2.0)*(kb/q)/dyr/ddh)*-dB(-xipr)*dxirpdThc*pc-(mupr*((Thr+Thc)/2.0)*(kb/q)/dyr/ddh)*dB(xipr)*dxirpdThc*pr);
	//dJprdTh+=-((mupr*((Thr+Thc)/2.0)*(kb/q)/dyr/ddh)*-dB(-xipr)*dxirpdThr*pc-(mupr*((Thr+Thc)/2.0)*(kb/q)/dyr/ddh)*dB(xipr)*dxirpdThr*pr);

	if (in->jouleheat==TRUE)
	{
		dJpldTh+=q*(Evr-Evl)*(mupl*(1.0/4.0)*(kb/q)/dyl/dh)*B(-xipl)*dpl;
		dJpcdTh+=-q*(Evr-Evl)*(mupl*(1.0/4.0)*(kb/q)/dyl/dh)*B(xipl)*dpc+q*(Evr-Evl)*(mupr*(1.0/4.0)*(kb/q)/dyr/dh)*B(-xipr)*dpc;
		dJprdTh+=-q*(Evr-Evl)*(mupr*(1.0/4.0)*(kb/q)/dyr/dh)*B(xipr)*dpr;

		dJpldTh+=q*(Evr-Evl)*(mupl*((Thc+Thl)/4.0)*(kb/q)/dyl/dh)*-dB(-xipl)*dxilpdThl*pl;
		dJpcdTh+=-q*(Evr-Evl)*(mupl*((Thc+Thl)/4.0)*(kb/q)/dyl/dh)*dB(xipl)*dxilpdThc*pc+q*(Evr-Evl)*(mupr*((Thr+Thc)/4.0)*(kb/q)/dyr/dh)*-dB(-xipr)*dxirpdThc*pc;
		dJprdTh+=-q*(Evr-Evl)*(mupr*((Thr+Thc)/4.0)*(kb/q)/dyr/dh)*dB(xipr)*dxirpdThr*pr;
	}

	///////////////////////////////////dTedphi////////////////////////////////

	double dsxilpdphil=q*2.0/(kb*(Thc+Thl));
	double dsxilpdphic=-q*2.0/(kb*(Thc+Thl));
	double dsxirpdphic=q*2.0/(kb*(Thr+Thc));
	double dsxirpdphir=-q*2.0/(kb*(Thr+Thc));

	dThldphi=Spl_pre*(-dB(-sxipl)*dsxilpdphil*pl-dB(sxipl)*dsxilpdphil*pc)/ddh;
	dThcdphi=Spl_pre*(-dB(-sxipl)*dsxilpdphic*pl-dB(sxipl)*dsxilpdphic*pc)/ddh-Spr_pre*(-dB(-sxipr)*dsxirpdphic*pc-B(sxipr)*dsxirpdphic*pr)/ddh;
	dThrdphi=-Spr_pre*(-dB(-sxipr)*dsxirpdphir*pc-B(sxipr)*dsxirpdphir*pr)/ddh;

	if (in->jouleheat==TRUE)
	{
		dThldphi+=q*(Jpr+Jpl)/2.0/dh;
		dThrdphi+=-q*(Jpr+Jpl)/2.0/dh;
	}

	//dphidTh=-q*(dpcdTh);  Does not like this one

	////////////////////dThdp////////////////////////////////////////////////
	dThldp=0.0;
	dThcdp=0.0;
	dThrdp=0.0;

	//Spl=Spl_pre*(B(-sxipl)*pl-B(sxipl)*pc);
	//Spr=Spr_pre*(B(-sxipr)*pc-B(sxipr)*pr);

	dThldp=Spl_pre*B(-sxipl)*dpl/ddh;
	dThcdp=-Spl_pre*(B(sxipl)*dpc)/ddh-Spr_pre*B(-sxipr)*dpc/ddh;
	dThrdp=Spr_pre*B(sxipr)*dpr/ddh;

	if (in->jouleheat==TRUE)
	{
		dThldp+=q*(Evr-Evl)*(Dpl/dyl/2.0/dh)*B(-xipl)*dpl;
		dThcdp+=-q*(Evr-Evl)*(Dpl/dyl/2.0/dh)*B(xipl)*dpc+q*(Evr-Evl)*(Dpr/dyr/2.0/dh)*B(-xipr)*dpc;
		dThrdp+=-q*(Evr-Evl)*(Dpr/dyr/2.0/dh)*B(xipr)*dpr;
	}
	dThcdp+=-(3.0/2.0)*kb*dpc*(Thc-Tlc)/in->tau_h;
}*/




/*
if (in->lnewton==TRUE)
{
	/////Calculate dTldTl////////
	dTll=kl0/dyl/ddh;
	dTlc=-(kl0/dyl/ddh+kl1/dyr/ddh);
	dTlr=kl1/dyr/ddh;
	dTl=dTll*Tll+dTlc*Tlc+dTlr*Tlr+(3.0/2.0)*kb*nc*(Tec-Tlc)/in->tau_e+(3.0/2.0)*kb*pc*(Thc-Tlc)/in->tau_h;

	///////////dTldTe//////////
	dTlcdTe=(3.0/2.0)*kb*nc/in->tau_e;
	///////////dTldTh//////////
	dTlcdTh=(3.0/2.0)*kb*pc/in->tau_h;


	///////////dTldTn//////////
	dTlcdn=(3.0/2.0)*kb*dnc*(Tec-Tlc)/in->tau_e;
	///////////dTldTp//////////
	dTlcdp=(3.0/2.0)*kb*dpc*(Thc-Tlc)/in->tau_h;

}
*/
			if (in->ntrapnewton==TRUE)
			{
				dntrapdntrap=-dntc/taunescapec;
				dntrapdn=dnc[0]/tauncapc;
				dntrap=nc[0]/tauncapc-ntc/taunescapec;
			}

			if (in->ptrapnewton==TRUE)
			{
				dptrapdptrap=-dptc/taupescapec;
				dptrapdp=dpc[0]/taupcapc;
				dptrap=pc[0]/taupcapc-ptc/taupescapec;
			}

			if (i!=0)
			{
				in->Ti[pos]=i;
				in->Tj[pos]=i-1;
				in->Tx[pos]=dphil;
				pos++;
				//electron
				for (band=0;band<bands;band++)
				{
					in->Ti[pos]=in->ymeshpoints*(1+band)+i;
					in->Tj[pos]=in->ymeshpoints*(1+band)+i-1;
					in->Tx[pos]=dJdxil[band];
					pos++;

					in->Ti[pos]=in->ymeshpoints*(1+band)+i;
					in->Tj[pos]=i-1;
					in->Tx[pos]=dJdphil[band];
					pos++;

					//hole
					in->Ti[pos]=in->ymeshpoints*(1+bands+band)+i;
					in->Tj[pos]=in->ymeshpoints*(1+bands+band)+i-1;
					in->Tx[pos]=dJpdxipl[band];
					pos++;

					in->Ti[pos]=i+in->ymeshpoints*(1+bands+band);
					in->Tj[pos]=i-1;
					in->Tx[pos]=dJpdphil[band];
					pos++;

				}

				/*if (in->enable_ex==TRUE)
				{
					//exiton
					in->Ti[pos]=i+3*in->ymeshpoints;
					in->Tj[pos]=i+3*in->ymeshpoints-1;
					in->Tx[pos]=dexl;
					pos++;


				}

				if (in->nnewton==TRUE)
				{
					//exiton
					in->Ti[pos]=i+4*in->ymeshpoints;
					in->Tj[pos]=i+4*in->ymeshpoints-1;
					in->Tx[pos]=dTel;
					pos++;

					in->Ti[pos]=i+4*in->ymeshpoints;
					in->Tj[pos]=i+in->ymeshpoints-1;
					in->Tx[pos]=dTeldn;
					pos++;

					in->Ti[pos]=i+in->ymeshpoints;
					in->Tj[pos]=i+4*in->ymeshpoints-1;
					in->Tx[pos]=dJnldTe;
					pos++;

					in->Ti[pos]=i+4*in->ymeshpoints;
					in->Tj[pos]=i-1;
					in->Tx[pos]=dTeldphi;
					pos++;
				}


				if (in->hnewton==TRUE)
				{

					in->Ti[pos]=i+5*in->ymeshpoints;
					in->Tj[pos]=i+5*in->ymeshpoints-1;
					in->Tx[pos]=dThl;
					pos++;

					in->Ti[pos]=i+5*in->ymeshpoints;
					in->Tj[pos]=i+2*in->ymeshpoints-1;
					in->Tx[pos]=dThldp;
					pos++;

					in->Ti[pos]=i+2*in->ymeshpoints;
					in->Tj[pos]=i+5*in->ymeshpoints-1;
					in->Tx[pos]=dJpldTh;
					pos++;

					in->Ti[pos]=i+5*in->ymeshpoints;
					in->Tj[pos]=i-1;
					in->Tx[pos]=dThldphi;
					pos++;
				}

				if (in->lnewton==TRUE)
				{
					in->Ti[pos]=i+6*in->ymeshpoints;
					in->Tj[pos]=i+6*in->ymeshpoints-1;
					in->Tx[pos]=dTll;
					pos++;
				}*/


			}

			//phi
			in->Ti[pos]=i;
			in->Tj[pos]=i;
			in->Tx[pos]=dphic;
			pos++;

			for (band=0;band<bands;band++)
			{
			in->Ti[pos]=i;
			in->Tj[pos]=i+in->ymeshpoints*(1+band);
			in->Tx[pos]=dphidxic[band];
			pos++;

			in->Ti[pos]=i;
			in->Tj[pos]=i+in->ymeshpoints*(1+bands+band);
			in->Tx[pos]=dphidxipc[band];
			pos++;
			}

			//electron
			for (band=0;band<bands;band++)
			{
			in->Ti[pos]=in->ymeshpoints*(1+band)+i;
			in->Tj[pos]=in->ymeshpoints*(1+band)+i;
			in->Tx[pos]=dJdxic[band];
			pos++;


			in->Ti[pos]=in->ymeshpoints*(1+band)+i;
			in->Tj[pos]=i;
			in->Tx[pos]=dJdphic[band];
			pos++;

			//Check me if we go multi band again
			//in->Ti[pos]=in->ymeshpoints*(1+band)+i;
			//in->Tj[pos]=in->ymeshpoints*(1+bands+band)+i;
			//in->Tx[pos]=dJdxipc;
			//pos++;
			}


			//hole
			for (band=0;band<bands;band++)
			{
			in->Ti[pos]=in->ymeshpoints*(1+bands+band)+i;
			in->Tj[pos]=in->ymeshpoints*(1+bands+band)+i;
			in->Tx[pos]=dJpdxipc[band];
			pos++;


			in->Ti[pos]=in->ymeshpoints*(1+bands+band)+i;
			in->Tj[pos]=i;
			in->Tx[pos]=dJpdphic[band];
			pos++;

			//Check me if we go multi band again
			//in->Ti[pos]=in->ymeshpoints*(1+bands+band)+i;
			//in->Tj[pos]=in->ymeshpoint*(1+band)+i;
			//in->Tx[pos]=dJpdxic;
			//pos++;
			}


			//exciton

			/*if (in->enable_ex==TRUE)
			{
				in->Ti[pos]=i+3*in->ymeshpoints;
				in->Tj[pos]=i+3*in->ymeshpoints;
				in->Tx[pos]=dexc;
				pos++;

				in->Ti[pos]=i+3*in->ymeshpoints;
				in->Tj[pos]=i+in->ymeshpoints;
				in->Tx[pos]=dexdn;
				pos++;

				in->Ti[pos]=i+3*in->ymeshpoints;
				in->Tj[pos]=i+2*in->ymeshpoints;
				in->Tx[pos]=dexdp;
				pos++;

				in->Ti[pos]=i+in->ymeshpoints;
				in->Tj[pos]=i+3*in->ymeshpoints;
				in->Tx[pos]=dJndex;
				pos++;

				in->Ti[pos]=i+2*in->ymeshpoints;
				in->Tj[pos]=i+3*in->ymeshpoints;
				in->Tx[pos]=dJpdex;
				pos++;

			}

			if (in->nnewton==TRUE)
			{
				//exiton
				in->Ti[pos]=i+4*in->ymeshpoints;
				in->Tj[pos]=i+4*in->ymeshpoints;
				in->Tx[pos]=dTec;
				pos++;

				in->Ti[pos]=i+4*in->ymeshpoints;
				in->Tj[pos]=i+in->ymeshpoints;
				in->Tx[pos]=dTecdn;
				pos++;


				in->Ti[pos]=i+in->ymeshpoints;
				in->Tj[pos]=i+4*in->ymeshpoints;
				in->Tx[pos]=dJncdTe;
				pos++;

				in->Ti[pos]=i;
				in->Tj[pos]=i+4*in->ymeshpoints;
				in->Tx[pos]=dphidTe;
				pos++;

				in->Ti[pos]=i+4*in->ymeshpoints;
				in->Tj[pos]=i;
				in->Tx[pos]=dTecdphi;
				pos++;

			}

			if (in->hnewton==TRUE)
			{
				in->Ti[pos]=i+5*in->ymeshpoints;
				in->Tj[pos]=i+5*in->ymeshpoints;
				in->Tx[pos]=dThc;
				pos++;

				in->Ti[pos]=i+5*in->ymeshpoints;
				in->Tj[pos]=i+2*in->ymeshpoints;
				in->Tx[pos]=dThcdp;
				pos++;

				in->Ti[pos]=i+2*in->ymeshpoints;
				in->Tj[pos]=i+5*in->ymeshpoints;
				in->Tx[pos]=dJpcdTh;
				pos++;

				in->Ti[pos]=i+5*in->ymeshpoints;
				in->Tj[pos]=i;
				in->Tx[pos]=dThcdphi;
				pos++;
			}

			if (in->lnewton==TRUE)
			{
				in->Ti[pos]=i+6*in->ymeshpoints;
				in->Tj[pos]=i+6*in->ymeshpoints;
				in->Tx[pos]=dTlc;
				pos++;

				in->Ti[pos]=i+6*in->ymeshpoints;
				in->Tj[pos]=i+4*in->ymeshpoints;
				in->Tx[pos]=dTlcdTe;
				pos++;

				in->Ti[pos]=i+6*in->ymeshpoints;
				in->Tj[pos]=i+5*in->ymeshpoints;
				in->Tx[pos]=dTlcdTh;
				pos++;

				in->Ti[pos]=i+6*in->ymeshpoints;
				in->Tj[pos]=i+in->ymeshpoints;
				in->Tx[pos]=dTlcdn;
				pos++;

				in->Ti[pos]=i+6*in->ymeshpoints;
				in->Tj[pos]=i+2*in->ymeshpoints;
				in->Tx[pos]=dTlcdp;
				pos++;

			}*/

			if (in->ntrapnewton==TRUE)
			{
				in->Ti[pos]=in->ymeshpoints*(1+bands+bands)+i;
				in->Tj[pos]=in->ymeshpoints*(1+bands+bands)+i;
				in->Tx[pos]=dntrapdntrap;
				pos++;

				in->Ti[pos]=in->ymeshpoints*(1+bands+bands)+i;
				in->Tj[pos]=in->ymeshpoints*1+i;
				in->Tx[pos]=dntrapdn;
				pos++;

				in->Ti[pos]=i;
				in->Tj[pos]=in->ymeshpoints*(1+bands+bands)+i;
				in->Tx[pos]=dphidntrap;
				pos++;


			}

			if (in->ptrapnewton==TRUE)
			{
				in->Ti[pos]=in->ymeshpoints*(1+bands+bands+1)+i;
				in->Tj[pos]=in->ymeshpoints*(1+bands+bands+1)+i;
				in->Tx[pos]=dptrapdptrap;
				pos++;

				in->Ti[pos]=in->ymeshpoints*(1+bands+bands+1)+i;
				in->Tj[pos]=in->ymeshpoints*(1+bands)+i;
				in->Tx[pos]=dptrapdp;
				pos++;

				in->Ti[pos]=i;
				in->Tj[pos]=in->ymeshpoints*(1+bands+bands+1)+i;
				in->Tx[pos]=dphidptrap;
				pos++;


			}

			if (i!=(in->ymeshpoints-1))
			{
				//phi
				in->Ti[pos]=i;
				in->Tj[pos]=i+1;
				in->Tx[pos]=dphir;
				pos++;


				for (band=0;band<bands;band++)
				{

				//electron
				in->Ti[pos]=in->ymeshpoints*(1+band)+i;
				in->Tj[pos]=in->ymeshpoints*(1+band)+i+1;
				in->Tx[pos]=dJdxir[band];
				pos++;

				in->Ti[pos]=i+in->ymeshpoints*(1+band);
				in->Tj[pos]=i+1;
				in->Tx[pos]=dJdphir[band];
				pos++;

				//hole
				in->Ti[pos]=in->ymeshpoints*(1+bands+band)+i;
				in->Tj[pos]=in->ymeshpoints*(1+bands+band)+i+1;
				in->Tx[pos]=dJpdxipr[band];
				pos++;

				in->Ti[pos]=i+in->ymeshpoints*(1+bands+band);
				in->Tj[pos]=i+1;
				in->Tx[pos]=dJpdphir[band];
				pos++;
				}


				//exiton
				/*if (in->enable_ex==TRUE)
				{
					in->Ti[pos]=i+3*in->ymeshpoints;
					in->Tj[pos]=i+3*in->ymeshpoints+1;
					in->Tx[pos]=dexr;
					pos++;
				}

				if (in->nnewton==TRUE)
				{
					in->Ti[pos]=i+4*in->ymeshpoints;
					in->Tj[pos]=i+4*in->ymeshpoints+1;
					in->Tx[pos]=dTer;
					pos++;

					in->Ti[pos]=i+4*in->ymeshpoints;
					in->Tj[pos]=i+in->ymeshpoints+1;
					in->Tx[pos]=dTerdn;
					pos++;


					in->Ti[pos]=i+in->ymeshpoints;
					in->Tj[pos]=i+4*in->ymeshpoints+1;
					in->Tx[pos]=dJnrdTe;
					pos++;

					in->Ti[pos]=i+4*in->ymeshpoints;
					in->Tj[pos]=i+1;
					in->Tx[pos]=dTerdphi;
					pos++;

				}

				if (in->hnewton==TRUE)
				{
					in->Ti[pos]=i+5*in->ymeshpoints;
					in->Tj[pos]=i+5*in->ymeshpoints+1;
					in->Tx[pos]=dThr;
					pos++;

					in->Ti[pos]=i+5*in->ymeshpoints;
					in->Tj[pos]=i+2*in->ymeshpoints+1;
					in->Tx[pos]=dThrdp;
					pos++;

					in->Ti[pos]=i+2*in->ymeshpoints;
					in->Tj[pos]=i+5*in->ymeshpoints+1;
					in->Tx[pos]=dJprdTh;
					pos++;

					in->Ti[pos]=i+5*in->ymeshpoints;
					in->Tj[pos]=i+1;
					in->Tx[pos]=dThrdphi;
					pos++;

				}

				if (in->lnewton==TRUE)
				{
					in->Ti[pos]=i+6*in->ymeshpoints;
					in->Tj[pos]=i+6*in->ymeshpoints+1;
					in->Tx[pos]=dTlr;
					pos++;
				}*/


			}

			//Possion
			in->b[i]=-(deriv-q*(-in->Nad[i]));
			for (band=0;band<bands;band++)
			{
				in->b[i]+=-(-q*(in->p[i][band]-in->n[i][band]));
			}

			//if ((in->ntrapnewton)||(in->combitrap))
			//{
			//	in->b[i]+=-(-q*(-ntc));
			//}

			//if ((in->ptrapnewton==TRUE)||(in->combitrap))
			//{
			//	in->b[i]+=-(-q*(ptc));
			//}

			for (band=0;band<bands;band++)
			{
			//electron
			in->b[in->ymeshpoints*(1+band)+i]=-((Jnr[band]-Jnl[band])/ddh-Rbi);
			//printf("%e\n",Rbi);
			if (in->go_time==TRUE) in->b[in->ymeshpoints*(1+band)+i]-=-(nc[0]-nlast)/dt;

			if (in->Dphoton==TRUE) in->b[in->ymeshpoints*(1+band)+i]-=in->Gex[i]*in->Dphotoneff;
//printf("%e %e\n",Rbi,in->Gex[i]*in->Dphotoneff);
			//hole
			in->b[in->ymeshpoints*(1+bands+band)+i]=-((Jpr[band]-Jpl[band])/ddh+Rbi);

			if (in->Dphoton==TRUE) in->b[in->ymeshpoints*(1+bands+band)+i]-=-in->Gex[i]*in->Dphotoneff;
			if (in->go_time==TRUE) in->b[in->ymeshpoints*(1+bands+band)+i]-=(pc[0]-plast)/dt;
			}
/*

			if (in->enable_ex==TRUE)
			{
				in->b[3*in->ymeshpoints+i]=-(dex);
			}

			if (in->nnewton==TRUE)
			{
				in->b[4*in->ymeshpoints+i]=-(dTe);
			}

			if (in->hnewton==TRUE)
			{
				in->b[5*in->ymeshpoints+i]=-(dTh);
			}

			if (in->lnewton==TRUE)
			{
				in->b[6*in->ymeshpoints+i]=-(dTl);
			}
*/
			if (in->ntrapnewton==TRUE)
			{
				in->b[in->ymeshpoints*(1+bands+bands)+i]=-(dntrap);
			}

			if (in->ptrapnewton==TRUE)
			{
				in->b[in->ymeshpoints*(1+bands+bands+1)+i]=-(dptrap);
			}

		if (pos>in->N)
		{
			printf("Error\n");
		}
		}


free(Jnl);
free(Jnr);
free(Jpl);
free(Jpr);

free(Dnl);
free(Dnc);
free(Dnr);
free(Dpl);
free(Dpc);
free(Dpr);

free(nl);
free(nc);
free(nr);

free(pl);
free(pc);
free(pr);

free(xil);
free(xir);
free(xipl);
free(xipr);

free(dJpdxipl);
free(dJpdxipc);
free(dJpdxipr);

free(dnl);
free(dnc);
free(dnr);

free(dpl);
free(dpc);
free(dpr);

free(munl);
free(munc);
free(munr);

free(mupl);
free(mupc);
free(mupr);


free(wnl);
free(wnc);
free(wnr);

free(wpl);
free(wpc);
free(wpr);

free(dnldTe);
free(dncdTe);
free(dnrdTe);


free(dpldTh);
free(dpcdTh);
free(dprdTh);

free(nc_all);
free(pc_all);
free(dnc_all);
free(dpc_all);


free(dJdxil);
free(dJdxic);
free(dJdxir);

free(dJdphil);
free(dJdphic);
free(dJdphir);


free(dJpdphil);
free(dJpdphic);
free(dJpdphir);

free(dphidxic);
free(dphidxipc);
//	solver_dump_matrix(in->M,pos,in->Ti,in->Tj, in->Tx,in->b);
fclose(file_j);
}

double get_cur_error(struct device *in)
{
int i;
double phi=0.0;
double n=0.0;
double p=0.0;
double x=0.0;
double te=0.0;
double th=0.0;
double tl=0.0;
double ttn=0.0;
double ttp=0.0;
int band;
int bands=in->bands;
for (i=0;i<in->ymeshpoints;i++)
{

#ifdef only
	if (only_update_thermal==FALSE)
	{
#endif
		phi+=mod(in->b[i]);
		for (band=0;band<in->bands;band++)
		{
			n+=mod(in->b[in->ymeshpoints*(1+band)+i]);
			p+=+mod(in->b[in->ymeshpoints*(1+bands+band)+i]);
		}

		if (in->ntrapnewton==TRUE)
		{
			ttn+=mod(in->b[in->ymeshpoints*(1+bands+bands)+i]);
		}

		if (in->ptrapnewton==TRUE)
		{
			ttp+=mod(in->b[in->ymeshpoints*(1+bands+bands+1)+i]);
		}

		if (in->enable_ex==TRUE)
		{
			x+=mod(in->b[3*in->ymeshpoints+i])/1e32;
		}

		if (in->nnewton==TRUE)
		{
			te+=mod(in->b[4*in->ymeshpoints+i]);
		}

#ifdef only

		}else
		{
#endif
		if (in->hnewton==TRUE)
		{
			th+=mod(in->b[5*in->ymeshpoints+i]);
		}

		if (in->lnewton==TRUE)
		{
			tl+=mod(in->b[6*in->ymeshpoints+i]);
		}


#ifdef only
	}
#endif
}

double tot=phi+n+p+x+te+th+tl+ttn+ttp;

if (isnan( tot))
{
	printf("%e %e %e %e %e %e %e %e %e\n",phi,n,p,x,te,th,tl,ttn,ttp);
	solver_dump_matrix(in->M,in->N,in->Ti,in->Tj, in->Tx,in->b);
	exit(0);
}

return tot;
}

void solver_get_memory(struct device *in)
{
int i;
int bands=in->bands;
in->N=in->ymeshpoints*3-2;	//Possion main

for (i=0;i<bands;i++)
{
	in->N+=in->ymeshpoints*3-2;	//Je main
	in->N+=in->ymeshpoints*3-2;	//Jh main
}

if (in->enable_ex==TRUE)
{
in->N+=in->ymeshpoints*3-2;	//ex main
in->N+=in->ymeshpoints;		//ex dn
in->N+=in->ymeshpoints;		//ex dp
in->N+=in->ymeshpoints;		//dJndex
in->N+=in->ymeshpoints;		//dJpdex

}

for (i=0;i<in->bands;i++)
{
	in->N+=in->ymeshpoints*3-2;	//dJe/phi
	in->N+=in->ymeshpoints*3-2;	//dJh/phi
}

for (i=0;i<in->bands;i++)
{
	in->N+=in->ymeshpoints;		//dphi/dn
	in->N+=in->ymeshpoints;		//dphi/dh
}

//in->N+=in->ymeshpoints;		//dJndp
//in->N+=in->ymeshpoints;		//dJpdn

in->M=in->ymeshpoints;	//Pos

for (i=0;i<in->bands;i++)
{
	in->M+=in->ymeshpoints;		//Je
	in->M+=in->ymeshpoints;		//Jh
}

if (in->enable_ex==TRUE)
{
in->M+=in->ymeshpoints;		//ex
}

if (in->ntrapnewton==TRUE)
{
in->N+=in->ymeshpoints;		//dntrapdn
in->N+=in->ymeshpoints;		//dntrapdntrap
in->N+=in->ymeshpoints;		//dphidntrap

in->M+=in->ymeshpoints;		//nt
}

if (in->ptrapnewton==TRUE)
{
in->N+=in->ymeshpoints;		//dptrapdp
in->N+=in->ymeshpoints;		//dptrapdptrap
in->N+=in->ymeshpoints;		//dphidptrap

in->M+=in->ymeshpoints;		//pt
}


if (in->nnewton==TRUE)
{
in->N+=in->ymeshpoints*3-2;	//dTedTe
in->N+=in->ymeshpoints*3-2;	//dTedphi
in->N+=in->ymeshpoints*3-2;	//dJedTe
in->N+=in->ymeshpoints;		//dphidTe
in->N+=in->ymeshpoints*3-2;	//dTedn

in->M+=in->ymeshpoints;		//Te

}

/*if (in->nnewton==TRUE)
{
in->N+=in->ymeshpoints*3-2;	//dTedTe
in->N+=in->ymeshpoints*3-2;	//dTedphi
in->N+=in->ymeshpoints*3-2;	//dJedTe
in->N+=in->ymeshpoints;		//dphidTe
in->N+=in->ymeshpoints*3-2;	//dTedn

in->M+=in->ymeshpoints;		//Te

}*/

if (in->hnewton==TRUE)
{
in->N+=in->ymeshpoints*3-2;	//dThdTh
in->N+=in->ymeshpoints*3-2;		//dThdphi
in->N+=in->ymeshpoints*3-2;	//dThdTp
in->N+=in->ymeshpoints*3-2;	//dJpdTh
in->M+=in->ymeshpoints;		//Th
}

if (in->lnewton==TRUE)
{
in->N+=in->ymeshpoints*3-2;	//dTldTl
in->N+=in->ymeshpoints;		//dTldTe
in->N+=in->ymeshpoints;		//dTldTh
in->N+=in->ymeshpoints;		//dTldnc
in->N+=in->ymeshpoints;		//dTldcp
in->M+=in->ymeshpoints;		//Tl
}

in->Ti =    malloc(in->N*sizeof(int));
in->Tj =    malloc(in->N*sizeof(int));
in->Tx = malloc(in->N*sizeof(double));
in->b = malloc(in->M*sizeof(double));
in->back_b = malloc(in->M*sizeof(double));


}

void solver_free_memory(struct device *in)
{
free(in->Ti);
free(in->Tj);
free(in->Tx);
free(in->b);
free(in->back_b);
}

void dump_for_plot(struct device *in)
{
int band;
int bands=in->bands;
if (in->enable_plot)
{
	int i;
	FILE *out =fopen ("./band.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
			fprintf(out,"%e %e %e %e %e %e %e %e\n",in->ymesh[i],in->Ec[i],in->Ev[i],in->Fi[i],in->Fn[i][0],in->Fp[i][0],in->Fnt[i],in->Fpt[i]);
	}

	fclose(out);

	out =fopen ("./phi.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
			fprintf(out,"%e %e\n",in->ymesh[i],in->phi[i]);
	}

	fclose(out);

	out =fopen ("./j.dat","w");
	double Jtot=0.0;
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);
		Jtot=0.0;

		for (band=0;band<bands;band++)
		{
			fprintf(out,"%e %e ",in->Jn[i][band],in->Jp[i][band]);
			Jtot+=in->Jn[i][band]+in->Jp[i][band];
		}

		fprintf(out,"%e \n",Jtot);

	}

	fclose(out);

	out =fopen ("./Fn.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);

		for (band=0;band<bands;band++)
		{
			fprintf(out,"%e ",in->Fn[i][band]);
		}

		fprintf(out,"\n");

	}

	fclose(out);

	out =fopen ("./Fp.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);

		for (band=0;band<bands;band++)
		{
			fprintf(out,"%e ",in->Fp[i][band]);
		}

		fprintf(out,"\n");

	}

	fclose(out);


	out =fopen ("./mu.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);

		for (band=0;band<bands;band++)
		{
			fprintf(out,"%e %e ",in->mun[i][band],in->mup[i][band]);
		}

		fprintf(out,"\n");

	}

	fclose(out);

	out =fopen ("./n.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);

		for (band=0;band<bands;band++)
		{
			fprintf(out,"%e %e ",in->n[i][band]-in->nt[i],in->p[i][band]-in->pt[i]);
		}

		fprintf(out,"\n");
	}

	fclose(out);


	out =fopen ("./trap.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e ",in->ymesh[i]);

		fprintf(out,"%e %e ",in->nt[i],in->pt[i]);

		fprintf(out,"\n");
	}

	fclose(out);


	out =fopen ("./Dex.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
			fprintf(out,"%e %e %e %e\n",in->ymesh[i],in->ex[i],in->G[i],in->Gex[i]);
	}

	fclose(out);

	out =fopen ("./R.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
			fprintf(out,"%e %e\n",in->ymesh[i],in->Rbi[i]);
	}

	if ((in->nnewton==TRUE)||(in->hnewton==TRUE))
	{
		dump_thermal(in);
	}
	fclose(out);

}
}

int solve_cur_thermal(struct device *in,int thermal)
{
//printf("I am here %d",thermal);
//getchar();
int ittr=0;
int cont=TRUE;



if (thermal==FALSE)
{

	solve_cur(in);
}else
{
		do
		{
			solve_cur(in);
			solve_thermal(in);
			//printf("ii=%d\n",ii);
			plot_now(in,"plot");
			///getchar();
		if (((in->thermal_conv==TRUE)&&(in->dd_conv==TRUE))||(ittr>10)) cont=FALSE;
		//printf("%d %d %d\n",in->thermal_conv,in->dd_conv,ii);
		//getchar();
		ittr++;
		}while(cont==TRUE);

}

return 0;
}

int solve_cur(struct device *in)
{
double error;
int ittr=0;
dprintf("Solve cur\n");




int doback=FALSE;
#ifdef only
only_update_thermal=FALSE;
#endif
//in->enable_back=FALSE;
int keep_going=FALSE;
int thermalrun=0;
	do
	{
		fill_matrix(in);
//dump_for_plot(in);
//plot_now(in,"plot");
	//solver_dump_matrix(in->M,in->N,in->Ti,in->Tj, in->Tx,in->b);
//getchar();
		if (in->enable_back)
		{
			if (ittr<5)
			{
				doback=TRUE;
			}else
			{
				doback=FALSE;
			}
		}
		//doback=FALSE;
		//if (ittr==6) doback=TRUE;

		//if (error<1e-6) doback=TRUE;

		doback=FALSE;
		//if (ittr<3) doback=FALSE;
		//doback=TRUE;
		//in->enable_back=FALSE;
		if (in->stop==TRUE)
		{
			break;
		}



		if (doback==TRUE)
		{
			do_backtrack(in);
			update_solver_vars(in,FALSE);
		}else
		{
			solver(in->M,in->N,in->Ti,in->Tj, in->Tx,in->b);
			update_solver_vars(in,TRUE);
			//getchar();
		}


		//thermalrun++;
		if (thermalrun==40) thermalrun=0;
		//update(in);
//getchar();
		error=get_cur_error(in);
		dprintf("%d Cur error = %e",ittr,error);
		if (doback==TRUE)
		{
			dprintf(" b");
		}


		dprintf("\n");
		ittr++;

		in->converge=fopen("./converge.dat","a");
		fprintf(in->converge,"%e\n",error);
		fclose(in->converge);

		keep_going=FALSE;

		quick_update();

		//printf("%le\n",in->min_cur_error);
		//getchar();
		in->dd_conv=FALSE;
		if ((ittr<2)&&(error<in->min_cur_error)) in->dd_conv=TRUE;
		if ((ittr<in->max_electrical_itt)&&(error>in->min_cur_error)) keep_going=TRUE;
	}while(keep_going==TRUE);
//getchar();
		//int r;
		//for (r=0;r<in->ymeshpoints;r++)
		//{
		//in->mun[r]=(get_mat_param_mobility(&(in->mat.l[in->ymatlayers[r]]),mat_mue,in->n[r])-in->mun[r])/100.0;
		//in->mup[r]=(get_mat_param_mobility(&(in->mat.l[in->ymatlayers[r]]),mat_muh,in->p[r])-in->mup[r])/100.0;
		//}
dump_for_plot(in);
plot_now(in,"plot");
in->odes+=in->M;
//getchar();
return 0;
}

