// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>

void black()
{
int i;
double E;
double Ts=3060;
double Estart=0.0e-19;//1e-7;
E=Estart;
int points=1000;
double Eend=5.0*1.6e-19;//2000e-9;
double dE=(Eend-Estart)/((double)points);
double *spec;
double b;
spec=malloc(sizeof(double)*points);
FILE *out;
out=fopen("./sun.dat","w");
for (i=0;i<points;i++)
{
	b=(2*pi*2.16e-5)/(pow(hp,3)*cl*cl);
	b*=E*E/(exp(E/(kb*Ts))-1);
	spec[i]=b;

	fprintf(out,"%e %e %e\n",E/1.6e-19,(hp*cl/E),b);
	E+=dE;
}
fclose(out);
}

void light_field(struct device *in,char *name)
{
double power=in->Psun*pow(10.0,-in->ND);
int lambda=100;
int i=0;
int l=0;
double E=0.0;
char material_name[100];
double I=0.0;
double photons=0.0;
double a=0.0;
FILE *out;
struct istruct sun;
struct istruct material;

strcpy(material_name,"./pcbmp3htspectra.dat");

int fit_len=get_file_len(name)-1;
inter_load(&sun,name,fit_len);
inter_rescale(&sun,1e-9, 1.0);
inter_norm(&sun,power);


fit_len=get_file_len(material_name)-1;
inter_load(&material,material_name,fit_len);
inter_rescale(&material,1e-9, 1e6);

double lstart=material.x[0];
double lstop=material.x[material.len-1];
double deltal=(lstop+(lstop*0.2)-lstart)/((double)lambda);
double lpos=lstart;

double d=in->ymesh[in->ymeshpoints-1];
double tot=0.0;
double x;
out=fopen("./spectra.dat","w");
FILE *out1d;
out1d=fopen("./spectra1d.dat","w");

FILE *out1dw;
out1dw=fopen("./spectra1dw.dat","w");

for (i=0;i<in->ymeshpoints;i++)
{
x=in->ymesh[i];
tot=0.0;
lpos=lstart;
	for (l=0;l<lambda;l++)
	{
		E=hp*cl/lpos;
		a=inter_get(&material,lpos);
		I=inter_get(&sun,lpos)*deltal/E;
		if (I<0) I=0.0;
		photons=a*I*(exp(-a*x)+exp(-2*a*d)*exp(a*x));
		//printf("%e %e %e %e %e %e %e\n",E,a,x,I,photons,lstart,lpos);
		//getchar();
		fprintf(out,"%e %e %e \n",lpos,x,photons);
		tot+=photons;
		lpos+=deltal;

		if (i==0) fprintf(out1dw,"%e %e\n",lpos,photons);

	}
fprintf(out,"\n");
fprintf(out1d,"%e %e\n",x,tot);
in->Gex[i]=tot;
}


fclose(out);
fclose(out1d);
fclose(out1dw);
inter_free(&material);
inter_free(&sun);
}


